// File: w32InetConnection.cpp 
//     Defines the w32InetConnection class for Windows socket use.
/* 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the WCTP Toolkit
 * 
 * The Initial Developer of the Original Code is WorldCom, Inc.
 * Portions created by WorldCom, Inc. are Copyright  2001
 * WorldCom, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * WorldCom, Inc. <skytel-strategic-apps@wcom.com> 
 * SEA, Inc. <jkarasz@seabase.com>
 * 
 */
/////////////////////////////////////////////////////////////////////////////

#ifndef W32INET_CONNECTION_HPP
#define W32INET_CONNECTION_HPP

#include <winsock.h>
#include <time.h>

#include "XString.hpp"

//
// Return error codes 
//
#define INET_CONN_OK 0
#define INET_CONN_OKAY 0
#define INET_CONN_ERROR 1
#define INET_CONN_CLOSE 2
#define INET_CONN_TIMEOUT 3
#define INET_CONN_CANCEL 4

//
// The message ID to use for winsock Async functions
// Note this is the same ID as used by MFC CSocket
//
#define INET_SOCKET_NOTIFY 0x0373

//
// Buffer size to use for reads and writes 
//
#define BUFFER_SIZE 2048

class w32InetConnection
{
public:
    // operations
    w32InetConnection();
    virtual ~w32InetConnection();

    int  open_conn(XString &error, const XString& hostname, int port);
    int  close_conn(XString& error);
    bool is_closed(void);

    int  writen(XString& error, const char *data, int length);
    int  readn(XString& error, XString &data, int length);
    int  read_available(XString& error, XString& data);

private:
    // operations
    void fast_close(void);
    int  ready_for_io(XString& error, 
					bool read_flag, bool write_flag, bool exp_flag);
    int  ready_for_read(XString& error);
    int  ready_for_write(XString& error);

    static int lookup_ipaddr(XString& error, const XString& hostname,
        struct sockaddr_in *addr);
    static XString translate_error(const char *funcname, int errnum);

	// attributes
    time_t             m_session_start;
    time_t             m_session_limit;
    int                m_usec_poll;
    struct sockaddr_in m_server_addr;
    SOCKET             m_sock_fd;
    char               m_buffer[BUFFER_SIZE];
};

#endif
