/* File:  w32WctpConnection.hpp
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the WCTP Toolkit
 * 
 * The Initial Developer of the Original Code is WorldCom, Inc.
 * Portions created by WorldCom, Inc. are Copyright  2001
 * WorldCom, Inc.  All Rights Reserved.
 * 
 * Contributor(s):
 * WorldCom, Inc. <skytel-strategic-apps@wcom.com> 
 * SEA, Inc. <jkarasz@seabase.com>
 */

#ifndef W32WCTP_CONNECTION_HPP
#define W32WCTP_CONNECTION_HPP

#include "w32InetConnection.hpp"

enum wctp_url_encode_t {
    WCTP_URL_ENCODE_NONE = 0,
    WCTP_URL_ENCODE_PART = 1,
    WCTP_URL_ENCODE_FULL = 2 
};

class w32WctpConnection : public w32InetConnection 
{
public:
    w32WctpConnection();
    ~w32WctpConnection();

    //////////////////////////////////
    // Send and Receive XML document 
    int 
    write_xml(XString &error, const XString &xml);
    int 
    read_xml(XString &error, XString &xml);

    //////////////////////////////////
    // What encoding to use on send
    void 
    set_send_urlencode(wctp_url_encode_t);
    wctp_url_encode_t 
    get_send_urlencode(void);

    ///////////////////////////////////
    // What encoding to use on receive
    void 
    set_receive_urlencode(wctp_url_encode_t);
    wctp_url_encode_t
    get_receive_urlencode(void);

// TEMP
// private:

    void wctp_encode(XString& dest, const XString& src);
    void wctp_decode(XString& dest, XString& src);

    bool is_tag_start_char(char ch);
    bool is_tag_char(char ch);
    int locate_char_plus(const XString& src, int start_pos, char ch);
    int locate_tag(const XString& src, int start_pos, 
        bool close_flag, XString& tag);
    int complete_xml(XString &dest, XString &src, XString &tag, int pos);

    XString m_leftover_raw;
    XString m_leftover_xml;
    wctp_url_encode_t m_send_urlencode;
    wctp_url_encode_t m_receive_urlencode;
};

#endif
