/* File:  wctpXmlClientMessage.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_CLIENT_MESSAGE_HPP
#define WCTP_XML_CLIENT_MESSAGE_HPP

#include "wctpXmlOperation.hpp"
#include "wctpXmlClientMessageReply.hpp"
#include "wctpXmlClientStatusInfo.hpp"

class wctpXmlClientMessage : public wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlClientMessage(); 
    wctpXmlClientMessage(const wctpXmlClientMessage &); 
    ~wctpXmlClientMessage(); 
    wctpXmlClientMessage& operator=(const wctpXmlClientMessage &);
    
    ////////////////////////////
    // Assemble XML protocol text
    virtual XString 
    AssembleOperation(XmlContext& ctx);

    ////////////////////////////
    // Parse XML protocol text
    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);

    ////////////////////////////
    // ClientMessage Type
    wctp_xml_t 
    GetClientMessageType(void);

    ////////////////////////////
    // ClientMessageReply Child Element (holds only one of set)
    wctpXmlClientMessageReply* 
    GetClientMessageReply(void);
    void // specify to hold this item from the set
    HoldClientMessageReply(void);

    ////////////////////////////
    // ClientStatusInfo Child Element (holds only one of set)
    wctpXmlClientStatusInfo* 
    GetClientStatusInfo(void);
    void // specify to hold this item from the set
    HoldClientStatusInfo(void);

private:
    virtual void Init(void);
    virtual void Copy(const wctpXmlClientMessage &);
    virtual void Free(void);
    
    wctp_parse_t 
    ParseObjectSwitch(XString& msg, XmlNode *pTree, int& pos);

    wctp_xml_t m_ClientMessageType;
    wctpXmlClientMessageReply* m_pClientMessageReply;
    wctpXmlClientStatusInfo* m_pClientStatusInfo;
 
};
#endif
