/* File:  wctpXmlClientQueryResponse.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_CLIENT_QUERY_RESPONSE_HPP
#define WCTP_XML_CLIENT_QUERY_RESPONSE_HPP

#include "wctpXmlClientMessage.hpp"
#include "wctpXmlNoMessages.hpp"
#include "wctpXmlFailure.hpp"
#include "wctpXmlOperation.hpp"

class wctpXmlClientQueryResponse : public wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlClientQueryResponse(); 
    wctpXmlClientQueryResponse(const wctpXmlClientQueryResponse &); 
    ~wctpXmlClientQueryResponse(); 
    wctpXmlClientQueryResponse& operator=(const wctpXmlClientQueryResponse &);

    ////////////////////////////
    // reset object to default state 
    virtual void 
    Clear(void);

    ////////////////////////////
    // ClientQueryResponse Type
    wctp_xml_t 
    GetClientQueryResponseType(void);

    ////////////////////////////
    // ClientMessage Child Elements (one or more)

    // erase all ClientMessage elements
    void 
    ClearClientMessage(void);  
    // get ClientMessage element at given position
    // ---- pos is a zero-based index 
    wctpXmlClientMessage* 
    GetClientMessage(int pos); 
    // add ClientMessage element at end of array
    wctpXmlClientMessage* 
    AddClientMessage(void);
    // erase ClientMessage element at given position
    // ---- pos is a zero-based index 
    void  
    EraseClientMessage(int pos);
    // return number of ClientMessage elements
    int 
    NumClientMessage(void);
    void // specify to hold this item from the set
    HoldClientMessage(void);

    ////////////////////////////
    // NoMessages Child Element (holds only one of set)
    wctpXmlNoMessages* 
    GetNoMessages(void);
    void // specify to hold this item from the set
    HoldNoMessages(void);

    ////////////////////////////
    // Failure Child Element (holds only one of set)
    wctpXmlFailure* 
    GetFailure(void);
    void // specify to hold this item from the set
    HoldFailure(void);

    ////////////////////////////
    // minNextPollInterval attribute
    const char * 
    GetMinNextPollInterval(void) const;
    void  
    SetMinNextPollInterval(const char *);

private:

    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlClientQueryResponse &);

    virtual void 
    Free(void);

    virtual XString 
    AssembleOperation(XmlContext& ctx);

    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);
    
    XString 
    AssembleClientMessage(XmlContext& ctx);

    wctp_parse_t 
    ParseClientMessage(XString& msg, XmlNode *pTree, int& pos);

    wctp_parse_t 
    ParseAttribute(XString& msg, const XString& key, const XString& val);

    wctp_parse_t 
    ParseObjectSwitch(XString& msg, XmlNode *pTree, int& pos);

    wctp_xml_t                   m_ClientQueryResponseType;
    TStack<wctpXmlClientMessage> m_vClientMessage;
    wctpXmlNoMessages           *m_pNoMessages;
    wctpXmlFailure              *m_pFailure;
    XString                      m_minNextPollInterval;
    
};
#endif
