/* File:  wctpXmlClientResponseHeader.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_CLIENT_RESPONSE_HEADER_HPP
#define WCTP_XML_CLIENT_RESPONSE_HEADER_HPP

#include "wctpXmlOperation.hpp"
#include "wctpXmlOriginator.hpp"
#include "wctpXmlRecipient.hpp"

class wctpXmlClientResponseHeader : public wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlClientResponseHeader(); 
    wctpXmlClientResponseHeader(const wctpXmlClientResponseHeader &); 
    ~wctpXmlClientResponseHeader(); 
    wctpXmlClientResponseHeader& operator=(const wctpXmlClientResponseHeader &);
    
    ////////////////////////////
    // Assemble XML protocol text
    virtual XString 
    AssembleOperation(XmlContext& ctx);

    ////////////////////////////
    // Parse XML protocol text
    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);
    
    ////////////////////////////
    // Originator Child Element (always holds)
    wctpXmlOriginator* 
    GetOriginator(void);

    ////////////////////////////
    // Recipient Child Element (always holds)
    wctpXmlRecipient* 
    GetRecipient(void);

    ////////////////////////////
    // respondingToTimestamp attribute
    const char * 
    GetRespondingToTimestamp(void) const;
    void  
    SetRespondingToTimestamp(const char *);

    ////////////////////////////
    // responseTimestamp attribute
    const char * 
    GetResponseTimestamp(void) const;
    void  
    SetResponseTimestamp(const char *);

private:
    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlClientResponseHeader &);

    virtual void 
    Free(void);

    wctp_parse_t 
    ParseAttribute(XString& msg, const XString& key, const XString& val);

    // attributes
    wctpXmlOriginator *m_pOriginator;
    wctpXmlRecipient  *m_pRecipient;
    XString            m_respondingToTimestamp;
    XString            m_responseTimestamp;
    
};
#endif
