/* File:  wctpXmlMessage.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_MESSAGE_HPP
#define WCTP_XML_MESSAGE_HPP

#include "wctpXmlSubmitRequest.hpp"
#include "wctpXmlMessageReply.hpp"
#include "wctpXmlStatusInfo.hpp"
#include "wctpXmlLookupResponse.hpp"

class wctpXmlMessage : public wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlMessage(); 
    wctpXmlMessage(const wctpXmlMessage &); 
    ~wctpXmlMessage(); 
    wctpXmlMessage& operator=(const wctpXmlMessage &);

    ////////////////////////////
    // Assemble XML protocol text
    virtual XString 
    AssembleOperation(XmlContext& ctx);

    ////////////////////////////
    // Parse XML protocol text
    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);
    
    ////////////////////////////
    // Message Type
    wctp_xml_t 
    GetMessageType(void);

    ////////////////////////////
    // SubmitRequest Child Element (holds only one of set)
    wctpXmlSubmitRequest* 
    GetSubmitRequest(void);
    void // specify to hold this item from the set
    HoldSubmitRequest(void);

    ////////////////////////////
    // MessageReply Child Element (holds only one of set)
    wctpXmlMessageReply* 
    GetMessageReply(void);
    void // specify to hold this item from the set
    HoldMessageReply(void);

    ////////////////////////////
    // StatusInfo Child Element (holds only one of set)
    wctpXmlStatusInfo* 
    GetStatusInfo(void);
    void // specify to hold this item from the set
    HoldStatusInfo(void);

    ////////////////////////////
    // LookupResponse Child Element (holds only one of set)
    wctpXmlLookupResponse* 
    GetLookupResponse(void);
    void // specify to hold this item from the set
    HoldLookupResponse(void);

    ////////////////////////////
    // sequenceNo attribute
    const char * 
    GetSequenceNo(void) const;
    void  
    SetSequenceNo(const char *);

private:
    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlMessage &);

    virtual void 
    Free(void);

    wctp_parse_t 
    ParseAttribute(XString& msg, const XString& key, const XString& val);

    wctp_parse_t 
    ParseObjectSwitch(XString& msg, XmlNode *pTree, int& pos);

    // attributes
    wctp_xml_t             m_MessageType;
    wctpXmlSubmitRequest  *m_pSubmitRequest;
    wctpXmlMessageReply   *m_pMessageReply;
    wctpXmlStatusInfo     *m_pStatusInfo;
    wctpXmlLookupResponse *m_pLookupResponse;
    XString                m_sequenceNo;
    
};
#endif
