/* File:  wctpXmlMessageControl.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_MESSAGE_CONTROL_HPP
#define WCTP_XML_MESSAGE_CONTROL_HPP

#include "wctpXmlOperation.hpp"

class wctpXmlMessageControl : public wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlMessageControl(); 
    wctpXmlMessageControl(const wctpXmlMessageControl &); 
    ~wctpXmlMessageControl(); 
    wctpXmlMessageControl& operator=(const wctpXmlMessageControl &);

    ////////////////////////////
    // Assemble XML protocol text
    virtual XString 
    AssembleOperation(XmlContext& ctx);

    ////////////////////////////
    // Parse XML protocol text
    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);
    
    ////////////////////////////
    // allowResponse attribute
    wctp_bool_t 
    GetAllowResponse(void) const;
    void  
    SetAllowResponse(wctp_bool_t);

    ////////////////////////////
    // allowTruncation attribute
    wctp_bool_t 
    GetAllowTruncation(void) const;
    void  
    SetAllowTruncation(wctp_bool_t);

    ////////////////////////////
    // deliveryAfter attribute
    const char * 
    GetDeliveryAfter(void) const;
    void  
    SetDeliveryAfter(const char *);

    ////////////////////////////
    // deliveryBefore attribute
    const char * 
    GetDeliveryBefore(void) const;
    void  
    SetDeliveryBefore(const char *);

    ////////////////////////////
    // deliveryPriority attribute
    wctp_priority_t 
    GetDeliveryPriority(void) const;
    void  
    SetDeliveryPriority(wctp_priority_t);

    ////////////////////////////
    // messageID attribute
    const char * 
    GetMessageID(void) const;
    void  
    SetMessageID(const char *);

    ////////////////////////////
    // notifyWhenDelivered attribute
    wctp_bool_t 
    GetNotifyWhenDelivered(void) const;
    void  
    SetNotifyWhenDelivered(wctp_bool_t);

    ////////////////////////////
    // notifyWhenQueued attribute
    wctp_bool_t 
    GetNotifyWhenQueued(void) const;
    void  
    SetNotifyWhenQueued(wctp_bool_t);

    ////////////////////////////
    // notifyWhenRead attribute
    wctp_bool_t 
    GetNotifyWhenRead(void) const;
    void  
    SetNotifyWhenRead(wctp_bool_t);

    ////////////////////////////
    // preformatted attribute
    wctp_bool_t 
    GetPreformatted(void) const;
    void  
    SetPreformatted(wctp_bool_t);

    ////////////////////////////
    // sendResponsesToID attribute
    const char * 
    GetSendResponsesToID(void) const;
    void  
    SetSendResponsesToID(const char *);

    ////////////////////////////
    // transactionID attribute
    const char * 
    GetTransactionID(void) const;
    void  
    SetTransactionID(const char *);

private:
    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlMessageControl &);

    wctp_parse_t 
    ParseAttribute(XString& msg, const XString& key, const XString& val);

    // attributes
    wctp_bool_t     m_allowResponse;
    wctp_bool_t     m_allowTruncation;
    XString         m_deliveryAfter;
    XString         m_deliveryBefore;
    wctp_priority_t m_deliveryPriority;
    XString         m_messageID;
    wctp_bool_t     m_notifyWhenDelivered;
    wctp_bool_t     m_notifyWhenQueued;
    wctp_bool_t     m_notifyWhenRead;
    wctp_bool_t     m_preformatted;
    XString         m_sendResponsesToID;
    XString         m_transactionID;
 
};
#endif
