/* File:  wctpXmlOperation.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_OPERATION_HPP
#define WCTP_XML_OPERATION_HPP

#include "wctpXml.hpp"
#include "XmlContext.hpp"
#include "XmlNode.hpp"

class wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlOperation(); 
    wctpXmlOperation(const wctpXmlOperation &); 
    
    virtual ~wctpXmlOperation(); 

    wctpXmlOperation& operator=(const wctpXmlOperation &);

    ////////////////////////////
    // reset object to default state 
    virtual void 
    Clear(void);

    ////////////////////////////
    // Assemble XML protocol text
    virtual const char* 
    AssembleRequestString(void);

    virtual const char* 
    Assemble(void);

    virtual XString
    Assemble(XmlContext& ctx);

    virtual XString
    AssembleOperation(XmlContext& ctx);

    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);

    //Assemble Operations
    virtual XString 
    AssembleOperationHeader(XmlContext& ctx);

    virtual XString 
    AssembleOperationFooter(XmlContext& ctx);

   ////////////////////////////
    // Parse XML protocol text
    virtual wctp_parse_t 
    Parse(const char *xml); 
 
    const char* 
    LastParseMessage(void);

    virtual XString& 
    GetTitle();

    void 
    SetTitle(const char * title);

    ////////////////////////////
    // Operation Type
    wctp_xml_t 
    GetOperationType(void);

    ////////////////////////////
    // wctpVersion attribute
    const char * 
    GetVersion(void) const;
    void  
    SetVersion(const char *);

    ////////////////////////////
    // wctpToken attribute
    const char * 
    GetToken(void) const;
    void  
    SetToken(const char *);

    ////////////////////////////
    // dtdFile attribute (SYSTEM location in xml header)
    const char * 
    GetDTDFile(void) const;
    void  
    SetDTDFile(const char *);

    void 
    SetPrefix(const char * newValue);


    ////////////////////////////
    // Set default values for static attributes for all future requests
    // THIS OPERATION IS NOT THREAD-SAFE: CALL IT BEFORE STARTING
    // YOUR THREADS. IF YOU CALL IT DURING THREAD EXECUTION, YOU
    // MAY GET INCONSISTENT HEADERS/ATTRIBUTES ON REQUESTS THAT 
    // ARE IN PROOGRESS AT THE TIME OF THE CHANGE.
    static void
    SetDefaultRequestValues(const char* wctp_version,
                            const char* dtd_file,
                            const char* wctp_token);

protected:
    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlOperation &);

    virtual void 
    Free(void);

    // Parse operations
    virtual wctp_parse_t 
    ParseHeader(XString& msg, XmlNode *pTree);

    virtual wctp_parse_t 
    ParseHeaderAttribute(XString& msg, const XString& key, const XString& val);
   
    void 
    LoadGlobals();
    
    // attributes
    wctp_xml_t m_OperationType;
    XString    m_xml;         // for holding Assembled XML string
    XString    m_parse_msg;   // for holding last Parse Message
    XString    m_xmlOperation;
    XString    m_xmlPrefix;
    XString    m_xmlVersion;
    XString    m_xmlToken;
    XString    m_title;
    XString    m_dtdFile;

private:
    // variables to be used as defaults values in every request sent. 
    // Can be overridden by using the instance-specific values above
    static XString g_wctp_version;
    static XString g_wctp_token;
    static XString g_dtd_file;

};
#endif
