/* File:  wctpXmlSubmitClientHeader.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_SUBMIT_CLIENT_HEADER_HPP
#define WCTP_XML_SUBMIT_CLIENT_HEADER_HPP

#include "wctpXmlOperation.hpp"
#include "wctpXmlClientOriginator.hpp"
#include "wctpXmlClientMessageControl.hpp"
#include "wctpXmlRecipient.hpp"

class wctpXmlSubmitClientHeader : wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlSubmitClientHeader(); 
    wctpXmlSubmitClientHeader(const wctpXmlSubmitClientHeader &); 
    ~wctpXmlSubmitClientHeader(); 
    wctpXmlSubmitClientHeader& operator=(const wctpXmlSubmitClientHeader &);

    ////////////////////////////
    // Assemble XML protocol text
    virtual XString 
    AssembleOperation(XmlContext& ctx);

    ////////////////////////////
    // Parse XML protocol text
    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);
    
    ////////////////////////////
    // ClientOriginator Child Element (always holds)
    wctpXmlClientOriginator* 
    GetClientOriginator(void);

    ////////////////////////////
    // ClientMessageControl Child Element (holds zero or one)
    wctpXmlClientMessageControl* 
    GetClientMessageControl(void);
    void
    UseClientMessageControl(void);
    void
    DontUseClientMessageControl(void);
    bool
    IsUsingClientMessageControl(void);
    void
    ClearClientMessageControl(void);

    ////////////////////////////
    // Recipient Child Element (always holds)
    wctpXmlRecipient* 
    GetRecipient(void);

    ////////////////////////////
    // submitTimestamp attribute
    const char * 
    GetSubmitTimestamp(void) const;
    void  
    SetSubmitTimestamp(const char *);

private:
    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlSubmitClientHeader &);

    virtual void 
    Free(void);

    wctp_parse_t 
    ParseAttribute(XString& msg, const XString& key, const XString& val);

    // attributes
    wctpXmlClientOriginator     *m_pClientOriginator;
    bool                         m_use_ClientMessageControl;
    wctpXmlClientMessageControl *m_pClientMessageControl;
    wctpXmlRecipient            *m_pRecipient;
    XString                      m_submitTimestamp;
    
 
};
#endif
