/* File:  wctpXmlSubmitHeader.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_SUBMIT_HEADER_HPP
#define WCTP_XML_SUBMIT_HEADER_HPP

#include "wctpXmlOperation.hpp"
#include "wctpXmlOriginator.hpp"
#include "wctpXmlMessageControl.hpp"
#include "wctpXmlRecipient.hpp"

class wctpXmlSubmitHeader : public wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlSubmitHeader(); 
    wctpXmlSubmitHeader(const wctpXmlSubmitHeader &); 
    ~wctpXmlSubmitHeader(); 
    wctpXmlSubmitHeader& operator=(const wctpXmlSubmitHeader &);

    ////////////////////////////
    // Assemble XML protocol text
    virtual XString 
    AssembleOperation(XmlContext& ctx);

    ////////////////////////////
    // Parse XML protocol text
    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);
    const char* 
    LastParseMessage(void);

    ////////////////////////////
    // Originator Child Element (always holds)
    wctpXmlOriginator* 
    GetOriginator(void);

    ////////////////////////////
    // MessageControl Child Element (always holds)
    wctpXmlMessageControl* 
    GetMessageControl(void);

    ////////////////////////////
    // Recipient Child Element (always holds)
    wctpXmlRecipient* 
    GetRecipient(void);

    ////////////////////////////
    // submitTimestamp attribute
    const char * 
    GetSubmitTimestamp(void) const;
    void  
    SetSubmitTimestamp(const char *);

private:
    void 
    Init(void);

    void 
    Copy(const wctpXmlSubmitHeader &);

    void 
    Free(void);
 
    wctp_parse_t 
    ParseAttribute(XString& msg, const XString& key, const XString& val);

    // attributes
    wctpXmlOriginator     *m_pOriginator;
    wctpXmlMessageControl *m_pMessageControl;
    wctpXmlRecipient      *m_pRecipient;
    XString                m_submitTimestamp;
    XString                m_xml;         // for holding Assembled XML string
    XString                m_parse_msg;   // for holding last Parse Message
 
};
#endif
