/* File:  wctpXmlTransparentData.hpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#ifndef WCTP_XML_TRANSPARENT_DATA_HPP
#define WCTP_XML_TRANSPARENT_DATA_HPP

#include "wctpXmlOperation.hpp"

class wctpXmlTransparentData : public wctpXmlOperation
{
public:
    ////////////////////////////
    // constructors, destructors, and assignemnt operators
    wctpXmlTransparentData(); 
    wctpXmlTransparentData(const wctpXmlTransparentData &); 
    ~wctpXmlTransparentData(); 
    wctpXmlTransparentData& operator=(const wctpXmlTransparentData &);

    ////////////////////////////
    // reset object to default state 
    void 
    Clear(void);

    ////////////////////////////
    // Assemble XML protocol text
    virtual XString 
    AssembleOperation(XmlContext& ctx);

    ////////////////////////////
    // Parse XML protocol text
    virtual wctp_parse_t 
    ParseObject(XString& msg, XmlNode *pTree);
    
    ////////////////////////////
    // encoding attribute
    wctp_encoding_t 
    GetEncoding(void) const;
    void  
    SetEncoding(wctp_encoding_t);

    ////////////////////////////
    // pcdata attribute
    const char * 
    GetPcdata(void) const;
    void  
    SetPcdata(const char *);
    int
    GetPcdataSize(void) const;
    void
    SetPcdata(const char *, int);
    const char *
    GetPcdataHexdump(void) const;

    ////////////////////////////
    // type attribute
    wctp_transparent_t 
    GetType(void) const;
    void  
    SetType(wctp_transparent_t);

private:
    virtual void 
    Init(void);

    virtual void 
    Copy(const wctpXmlTransparentData &);

    virtual void 
    Free(void);
    
    wctp_parse_t 
    ParseAttribute(XString& msg, const XString& key, const XString& val);

    // attributes
    wctp_encoding_t    m_encoding;
    XString            m_pcdata;
    XString            m_pcdata_hexdump;
    wctp_transparent_t m_type;
    
};
#endif
