/* File:  XmlNode.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include <iostream.h>
#include <stdio.h>
#include <ctype.h>

#include "XmlNode.hpp"


//////////////////////////////////////////////////////////////////////////////
//

XmlNode::XmlNode()
{
}

//////////////////////////////////////////////////////////////////////////////
//

XmlNode::~XmlNode()
{
    // printf("~XmlNode: %s\n", element.c_str());
    attributes.destroy();
    children.destroy();
}

//////////////////////////////////////////////////////////////////////////////
//

void 
XmlNode::start_cb(void *pData, const char *el, const char **attr) 
{
    TStack<XmlNode> *pStack = (TStack<XmlNode> *)pData;
    XmlNode *pNode = new XmlNode;
    pNode->element = el;
    pStack->push(pNode);

    int i;
    for(i=0; attr[i] != NULL && attr[i+1] != NULL; i+=2)
    {
        XmlAttribute *pAttribute = new XmlAttribute;
        pAttribute->key = attr[i];
        pAttribute->val = attr[i+1];
        pNode->attributes.push(pAttribute); 
    }
}

//////////////////////////////////////////////////////////////////////////////
//

void 
XmlNode::end_cb(void *pData, const char *el) 
{
    TStack<XmlNode> *pStack = (TStack<XmlNode> *)pData;
    if(pStack->size() > 1)
    {
        XmlNode *pChild = pStack->pop();
        XmlNode *pParent = pStack->top();
        pParent->children.push(pChild);
    }
}

//////////////////////////////////////////////////////////////////////////////
//

void 
XmlNode::char_cb(void *pData, const char *txt, int txtlen) 
{
    // collect pcdata 8-bit clean...
    TStack<XmlNode> *pStack = (TStack<XmlNode> *)pData;
    XString s(txt,txtlen); 
    if(!pStack->empty())
    {
        XmlNode *pNode = pStack->top();
        pNode->pcdata += s;
    }
}

//////////////////////////////////////////////////////////////////////////////
//

void 
XmlNode::dump_tree(XmlNode *pTree, int depth)
{
    // printf("%d: %s\n", depth, pTree->element.c_str());

    if(pTree->pcdata.length()>0)
    {
        // printf("    pcdata: |%s|\n", pTree->pcdata.c_str());
    }
    
    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XString key = pTree->attributes[i]->key;
        XString val = pTree->attributes[i]->val;
        // printf("    %s=|%s|\n", key.c_str(), val.c_str());
    }

    for(i=0; i<pTree->children.size(); i++)
    {  
        dump_tree(pTree->children[i], depth+1);
    }
}

//////////////////////////////////////////////////////////////////////////////
//

XmlNode*
XmlNode::parse(XString& msg, const XString &xml, XML_Parser& p)
{
    XmlNode *pTree = NULL;

    TStack<XmlNode> workStack;

    //XML_Parser p = XML_ParserCreate(NULL);
    p = XML_ParserCreate(NULL);
    XML_SetElementHandler(p, XmlNode::start_cb, XmlNode::end_cb);
    XML_SetCharacterDataHandler(p, XmlNode::char_cb);
    XML_SetUserData(p, (void*)&workStack);

    int done = 1;
    int flag = XML_Parse(p, xml.data(), xml.size(), done);

    if(flag && workStack.size() == 1)
    {
        XmlNode *pTree = workStack.pop();
        // dump_tree(pTree,0); 
        return pTree;
    }
    else if(!flag)
    {
        char buf[16];
        msg = "Parse error at line ";
        sprintf(buf,"%d", XML_GetCurrentLineNumber(p));
        msg += buf;
        msg += ",  ";
        msg += XML_ErrorString(XML_GetErrorCode(p));
    }
    else
    {
        msg = "internal error, related to expat";
    }

    workStack.destroy();
    XML_ParserFree(p);

    
    return pTree;
}

