/* File:  wctpXml.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXml.hpp"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

//GLobal variables from calling program
extern const char *g_wctp_version;
extern const char *g_wctp_dtd;

//////////////////////////////////////////////////////////////////////////////
// Generates a timestamp (not thread-safe!)

const char *
wctpXml::GenerateTimestamp(void)
{
    static char bufTime[64];
    struct tm *pTime;
    time_t iTime;
    time(&iTime);
    pTime = gmtime(&iTime);
    sprintf(bufTime, "%04d-%02d-%02dT%02d:%02d:%02d",
        pTime->tm_year+1900, pTime->tm_mon+1, pTime->tm_mday,
        pTime->tm_hour, pTime->tm_min, pTime->tm_sec);
    return bufTime;
}

//////////////////////////////////////////////////////////////////////////////
// Generates a message id (not thread-safe!)

const char *
wctpXml::GenerateMessageID(void)
{
    static char bufMessageID[64];
    static unsigned short counter = 0;
    time_t iTime;
    time(&iTime);
    sprintf(bufMessageID,"%u-%u", 
        (unsigned int)iTime, (unsigned int)counter);
    counter++;
    return bufMessageID;
}

//////////////////////////////////////////////////////////////////////////////
// convert wctp_bool_t to string

const char *
wctpXml::bool_to_str(wctp_bool_t val)
{
    char *res = "";
    switch(val)
    {
    case WCTP_BOOL_FALSE:
        res = "false";
        break;
    case WCTP_BOOL_TRUE:
        res = "true";
        break;
    case WCTP_BOOL_DEFAULT:
        res = "";
        break;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// convert string to wctp_bool_t 

wctp_bool_t
wctpXml::str_to_bool(const char * val)
{
    wctp_bool_t res = WCTP_BOOL_FALSE;
    if(XString::icmp(val,"true") == 0)
    {
        res = WCTP_BOOL_TRUE;
    }
    else if(XString::icmp(val,"") == 0)
    {
        res = WCTP_BOOL_DEFAULT;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// convert wctp_encoding_t to string

const char *
wctpXml::encoding_to_str(wctp_encoding_t val)
{
    char *res = "";
    switch(val)
    {
    case WCTP_ENCODING_DEFAULT:
        res = "";
        break;
    case WCTP_ENCODING_STANDARD:
        res = "standard";
        break;
    case WCTP_ENCODING_BASE64:
        res = "base64";
        break;
    case WCTP_ENCODING_UUENCODE:
        res = "uuencode";
        break;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// convert string to wctp_encoding_t 

wctp_encoding_t
wctpXml::str_to_encoding(const char * val)
{
    wctp_encoding_t res = WCTP_ENCODING_DEFAULT;
    if(XString::icmp(val,"standard") == 0)
    {
        res = WCTP_ENCODING_STANDARD;
    }
    else if(XString::icmp(val,"base64") == 0)
    {
        res = WCTP_ENCODING_BASE64;
    }
    else if(XString::icmp(val,"uuencode") == 0)
    {
        res = WCTP_ENCODING_UUENCODE;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// convert wctp_format_t to string

const char *
wctpXml::format_to_str(wctp_format_t val)
{
    char *res = "";
    switch(val)
    {
    case WCTP_FORMAT_DEFAULT:
        res = "";
        break;
    case WCTP_FORMAT_ALPHANUMERIC:
        res = "Alphanumeric";
        break;
    case WCTP_FORMAT_TRANSPARENT_DATA:
        res = "TransparentData";
        break;
    case WCTP_FORMAT_ALL:
        res = "All";
        break;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// convert string to wctp_format_t 

wctp_format_t
wctpXml::str_to_format(const char * val)
{
    wctp_format_t res = WCTP_FORMAT_DEFAULT;
    if(XString::icmp(val,"Alphanumeric") == 0)
    {
        res = WCTP_FORMAT_ALPHANUMERIC;
    }
    else if(XString::icmp(val,"TransparentData") == 0)
    {
        res = WCTP_FORMAT_TRANSPARENT_DATA;
    }
    else if(XString::icmp(val,"All") == 0)
    {
        res = WCTP_FORMAT_ALL;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// convert wctp_notification_t to string

const char *
wctpXml::notification_to_str(wctp_notification_t val)
{
    char *res = "";
    switch(val)
    {
    case WCTP_NOTIFICATION_DEFAULT:
        res = "";
        break;
    case WCTP_NOTIFICATION_QUEUED:
        res = "QUEUED";
        break;
    case WCTP_NOTIFICATION_DELIVERED:
        res = "DELIVERED";
        break;
    case WCTP_NOTIFICATION_READ:
        res = "READ";
        break;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// convert string to wctp_notification_t 

wctp_notification_t
wctpXml::str_to_notification(const char * val)
{
    wctp_notification_t res = WCTP_NOTIFICATION_DEFAULT;
    if(XString::icmp(val,"QUEUED") == 0)
    {
        res = WCTP_NOTIFICATION_QUEUED;
    }
    else if(XString::icmp(val,"DELIVERED") == 0)
    {
        res = WCTP_NOTIFICATION_DELIVERED;
    }
    else if(XString::icmp(val,"READ") == 0)
    {
        res = WCTP_NOTIFICATION_READ;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// convert wctp_priority_t to string

const char *
wctpXml::priority_to_str(wctp_priority_t val)
{
    char *res = "";
    switch(val)
    {
    case WCTP_PRIORITY_DEFAULT:
        res = "";
        break;
    case WCTP_PRIORITY_LOW:
        res = "LOW";
        break;
    case WCTP_PRIORITY_NORMAL:
        res = "NORMAL";
        break;
    case WCTP_PRIORITY_HIGH:
        res = "HIGH";
        break;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// convert string to wctp_priority_t 

wctp_priority_t
wctpXml::str_to_priority(const char * val)
{
    wctp_priority_t res = WCTP_PRIORITY_DEFAULT;
    if(XString::icmp(val,"LOW") == 0)
    {
        res = WCTP_PRIORITY_LOW;
    }
    else if(XString::icmp(val,"NORMAL") == 0)
    {
        res = WCTP_PRIORITY_NORMAL;
    }
    else if(XString::icmp(val,"HIGH") == 0)
    {
        res = WCTP_PRIORITY_HIGH;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// convert wctp_transparent_t to string

const char *
wctpXml::transparent_to_str(wctp_transparent_t val)
{
    char *res = "";
    switch(val)
    {
    case WCTP_TRANSPARENT_DEFAULT:
        res = "";
        break;
    case WCTP_TRANSPARENT_OPAQUE:
        res = "OPAQUE";
        break;
    case WCTP_TRANSPARENT_FLEXSUITE:
        res = "FLEXSuite";
        break;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// convert string to wctp_transparent_t 

wctp_transparent_t
wctpXml::str_to_transparent(const char * val)
{
    wctp_transparent_t res = WCTP_TRANSPARENT_DEFAULT;
    if(XString::icmp(val,"OPAQUE") == 0)
    {
        res = WCTP_TRANSPARENT_OPAQUE;
    }
    else if(XString::icmp(val,"FLEXSuite") == 0)
    {
        res = WCTP_TRANSPARENT_FLEXSUITE;
    }
    return res;
}


