/* File:  wctpXmlChoice.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlChoice.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlChoice::wctpXmlChoice()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlChoice::wctpXmlChoice(
    const wctpXmlChoice &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlChoice::~wctpXmlChoice()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlChoice&
wctpXmlChoice::operator=(
    const wctpXmlChoice &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlChoice::Init(void)
{
    wctpXmlOperation::Init();

    m_title = CHOICE_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlChoice::Copy(const wctpXmlChoice &rval)
{
    //
    // Copy member variables
    //
    wctpXmlOperation::Copy(rval);

    m_pcdata = rval.m_pcdata;

}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlChoice::AssembleOperation(XmlContext& ctx)
{
    XString dest;

    XString temp = "<";
    temp += m_xmlOperation;
    temp += ">";
    dest += ctx.OpenIndent(temp);

    dest += ctx.XmlPcdata(m_pcdata);

    temp = "</";
    temp += m_xmlOperation;
    temp += ">";
    dest += ctx.CloseIndent(temp);
    
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlChoice::ParseObject(XString& msg,XmlNode *pTree)
{
   
    //
    // make sure the element matches me
    //

    if(m_xmlOperation != pTree->element)
    {
        msg = "element mismatch, expected ";
        msg += m_xmlOperation;
        msg += ", but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }

    //
    // make sure no attributes where specified 
    //

    if(pTree->attributes.size() > 0)
    {
        msg = m_xmlOperation;
        msg += " does not have any attributes";
        return WCTP_PARSE_ERROR;
    }

    //
    // collect pcdata 
    //

    m_pcdata = pTree->pcdata.both_trim();

    //
    // Should not be any child objects
    //

    if(pTree->children.size() > 0)
    {
        msg = m_xmlOperation;
        msg += " element does not accept any child elements";
        return WCTP_PARSE_ERROR;
    }
    
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_pcdata attribute

const char * 
wctpXmlChoice::GetPcdata(void) const 
{
    return m_pcdata;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_pcdata attribute

void  
wctpXmlChoice::SetPcdata(const char * newValue)
{
    m_pcdata = newValue;
}


