/* File:  wctpXmlClientMessage.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include <stdio.h>
#include "wctpXmlClientMessage.hpp"

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlClientMessage::wctpXmlClientMessage()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlClientMessage::wctpXmlClientMessage(
    const wctpXmlClientMessage &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlClientMessage::~wctpXmlClientMessage()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlClientMessage&
wctpXmlClientMessage::operator=(
    const wctpXmlClientMessage &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlClientMessage::Init(void)
{
    wctpXmlOperation::Init();

    m_ClientMessageType = WCTP_XML_UNDEFINED;
    m_pClientMessageReply = NULL;
    m_pClientStatusInfo = NULL;

    m_title = CLIENT_MESSAGE_STRING;
    
       m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlClientMessage::Copy(const wctpXmlClientMessage &rval)
{
    //
    // Copy object type
    //
    wctpXmlOperation::Copy(rval);

    m_ClientMessageType = rval.m_ClientMessageType;

    //
    // Copy pClientMessageReply object if available (maybe holds)
    //

    if(rval.m_pClientMessageReply != NULL)
    {
        m_pClientMessageReply = new wctpXmlClientMessageReply(
            *rval.m_pClientMessageReply);
    }
    else
    {
        m_pClientMessageReply = NULL;
    }

    //
    // Copy pClientStatusInfo object if available (maybe holds)
    //

    if(rval.m_pClientStatusInfo != NULL)
    {
        m_pClientStatusInfo = new wctpXmlClientStatusInfo(
            *rval.m_pClientStatusInfo);
    }
    else
    {
        m_pClientStatusInfo = NULL;
    }

}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlClientMessage::Free(void)
{
    wctpXmlOperation::Free();

    if(m_pClientMessageReply != NULL) 
    {
        delete m_pClientMessageReply;
        m_pClientMessageReply = NULL;
    }
    if(m_pClientStatusInfo != NULL) 
    {
        delete m_pClientStatusInfo;
        m_pClientStatusInfo = NULL;
    }
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlClientMessage::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-ClientMessage>");
    if(m_ClientMessageType == WCTP_XML_CLIENT_MESSAGE_REPLY && 
       m_pClientMessageReply != NULL)
    {
        dest += m_pClientMessageReply->AssembleOperation(ctx);
    }
    if(m_ClientMessageType == WCTP_XML_CLIENT_STATUS_INFO && 
       m_pClientStatusInfo != NULL)
    {
        dest += m_pClientStatusInfo->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-ClientMessage>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse switch for choice among set of objects (must match one)

wctp_parse_t 
wctpXmlClientMessage::ParseObjectSwitch(
    XString& msg,
    XmlNode *pTree, 
    int& pos)
{
    wctp_parse_t res = WCTP_PARSE_ERROR;
    if(pos >= pTree->children.size())
    {
        msg = "An element is required inside wctp-ClientMessage";
        return res;
    }
    XmlNode *pChild = pTree->children[pos];
    if(pChild->element == "wctp-ClientMessageReply")
    {
        HoldClientMessageReply();
        pos++;
        res = m_pClientMessageReply->ParseObject(msg,pChild);
    }
    else if(pChild->element == "wctp-ClientStatusInfo")
    {
        HoldClientStatusInfo();
        pos++;
        res = m_pClientStatusInfo->ParseObject(msg,pChild);
    }
    else 
    {
        msg = "unexpected element |";
        msg += pChild->element;
        msg += "| found inside wctp-ClientMessage";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlClientMessage::ParseObject(XString& msg, XmlNode *pTree)
{
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-ClientMessage")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-ClientMessage, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }

    //
    // make sure no attributes where specified 
    //

    if(pTree->attributes.size() > 0)
    {
        msg = "wctp-ClientMessage does not have any attributes";
        return WCTP_PARSE_ERROR;
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-ClientMessage does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    int pos = 0;
    wctp_parse_t res;

    //
    // Load in an object from a set
    //

    res = ParseObjectSwitch(msg, pTree, pos);
    if(res != WCTP_PARSE_OKAY)
    { 
        return res; 
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-ClientMessage element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Accessor to find which object type is contained

wctp_xml_t 
wctpXmlClientMessage::GetClientMessageType(void)
{
    return m_ClientMessageType;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to ClientMessageReply object
// This function will never return NULL unless new fails...

wctpXmlClientMessageReply*
wctpXmlClientMessage::GetClientMessageReply(void) 
{
    if(m_pClientMessageReply == NULL)
    {
        m_pClientMessageReply = new wctpXmlClientMessageReply;
    }
    return m_pClientMessageReply;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold ClientMessageReply object 

void
wctpXmlClientMessage::HoldClientMessageReply(void) 
{
    m_ClientMessageType = WCTP_XML_CLIENT_MESSAGE_REPLY;
    if(m_pClientMessageReply == NULL)
    {
        m_pClientMessageReply = new wctpXmlClientMessageReply;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to ClientStatusInfo object
// This function will never return NULL unless new fails...

wctpXmlClientStatusInfo*
wctpXmlClientMessage::GetClientStatusInfo(void) 
{
    if(m_pClientStatusInfo == NULL)
    {
        m_pClientStatusInfo = new wctpXmlClientStatusInfo;
    }
    return m_pClientStatusInfo;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold ClientStatusInfo object 

void
wctpXmlClientMessage::HoldClientStatusInfo(void) 
{
    m_ClientMessageType = WCTP_XML_CLIENT_STATUS_INFO;
    if(m_pClientStatusInfo == NULL)
    {
        m_pClientStatusInfo = new wctpXmlClientStatusInfo;
    }
}

