/* File:  wctpXmlClientMessageReply.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlClientMessageReply.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlClientMessageReply::wctpXmlClientMessageReply()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlClientMessageReply::wctpXmlClientMessageReply(
    const wctpXmlClientMessageReply &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlClientMessageReply::~wctpXmlClientMessageReply()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlClientMessageReply&
wctpXmlClientMessageReply::operator=(
    const wctpXmlClientMessageReply &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlClientMessageReply::Init(void)
{
    wctpXmlOperation::Init();

    m_pClientResponseHeader = new wctpXmlClientResponseHeader;
    m_pPayload = new wctpXmlPayload;
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlClientMessageReply::Copy(const wctpXmlClientMessageReply &rval)
{
    //
    // Copy pClientResponseHeader object if available (should always hold one)
    //

    wctpXmlOperation::Copy(rval);

    if(rval.m_pClientResponseHeader != NULL)
    {
        m_pClientResponseHeader = new wctpXmlClientResponseHeader(
            *rval.m_pClientResponseHeader);
    }
    else
    {
        m_pClientResponseHeader = new wctpXmlClientResponseHeader;
    }

    //
    // Copy pPayload object if available (should always hold one)
    //

    if(rval.m_pPayload != NULL)
    {
        m_pPayload = new wctpXmlPayload(
            *rval.m_pPayload);
    }
    else
    {
        m_pPayload = new wctpXmlPayload;
    }

}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlClientMessageReply::Free(void)
{
    wctpXmlOperation::Free();

    if(m_pClientResponseHeader != NULL) 
    {
        delete m_pClientResponseHeader;
        m_pClientResponseHeader = NULL;
    }
    if(m_pPayload != NULL) 
    {
        delete m_pPayload;
        m_pPayload = NULL;
    }
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlClientMessageReply::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-ClientMessageReply>");
    if(m_pClientResponseHeader != NULL)
    {
        dest += m_pClientResponseHeader->AssembleOperation(ctx);
    }
    if(m_pPayload != NULL)
    {
        dest += m_pPayload->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-ClientMessageReply>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlClientMessageReply::ParseObject(XString& msg,XmlNode *pTree)
{
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-ClientMessageReply")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-ClientMessageReply, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }

    //
    // make sure no attributes where specified 
    //

    if(pTree->attributes.size() > 0)
    {
        msg = "wctp-ClientMessageReply does not have any attributes";
        return WCTP_PARSE_ERROR;
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-ClientMessageReply does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    int pos = 0;
    wctp_parse_t res;

    //
    // Load in wctp-ClientResponseHeader object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pClientResponseHeader->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-ClientResponseHeader element expected in ";
        msg += "wctp-ClientMessageReply element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in wctp-Payload object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pPayload->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Payload element expected in ";
        msg += "wctp-ClientMessageReply element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-ClientMessageReply element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to ClientResponseHeader object
// This function will never return NULL unless new fails...

wctpXmlClientResponseHeader*
wctpXmlClientMessageReply::GetClientResponseHeader(void) 
{
    if(m_pClientResponseHeader == NULL)
    {
        m_pClientResponseHeader = new wctpXmlClientResponseHeader;
    }
    return m_pClientResponseHeader;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Payload object
// This function will never return NULL unless new fails...

wctpXmlPayload*
wctpXmlClientMessageReply::GetPayload(void) 
{
    if(m_pPayload == NULL)
    {
        m_pPayload = new wctpXmlPayload;
    }
    return m_pPayload;
}

