/* File:  wctpXmlClientQueryResponse.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlClientQueryResponse.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlClientQueryResponse::wctpXmlClientQueryResponse()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlClientQueryResponse::wctpXmlClientQueryResponse(
    const wctpXmlClientQueryResponse &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlClientQueryResponse::~wctpXmlClientQueryResponse()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlClientQueryResponse&
wctpXmlClientQueryResponse::operator=(
    const wctpXmlClientQueryResponse &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlClientQueryResponse::Init(void)
{
    wctpXmlOperation::Init();

    m_OperationType = WCTP_XML_CLIENT_QUERY_RESPONSE;

    m_ClientQueryResponseType = WCTP_XML_UNDEFINED;
    m_pNoMessages = NULL;
    m_pFailure = NULL;
    m_minNextPollInterval = "";

    m_title = CLIENT_QUERY_RESPONSE_STRING;
    
    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;

}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlClientQueryResponse::Copy(const wctpXmlClientQueryResponse &rval)
{
    wctpXmlOperation::Copy(rval);

    //
    // Copy object type
    //

    m_ClientQueryResponseType = rval.m_ClientQueryResponseType;

    //
    // Copy pClientMessage object(s) if available (holds one or more)
    //

    if(rval.m_vClientMessage.size() > 0)
    {
        int i;
        for(i=0; i<rval.m_vClientMessage.size(); i++)
        {
            wctpXmlClientMessage *pClientMessage = 
                new wctpXmlClientMessage(
                *(rval.m_vClientMessage[i]));
            m_vClientMessage.push(pClientMessage);
        }
    }

    //
    // Copy pNoMessages object if available (maybe holds)
    //

    if(rval.m_pNoMessages != NULL)
    {
        m_pNoMessages = new wctpXmlNoMessages(
            *rval.m_pNoMessages);
    }
    else
    {
        m_pNoMessages = NULL;
    }

    //
    // Copy pFailure object if available (maybe holds)
    //

    if(rval.m_pFailure != NULL)
    {
        m_pFailure = new wctpXmlFailure(
            *rval.m_pFailure);
    }
    else
    {
        m_pFailure = NULL;
    }

    //
    // Copy member variables
    //

    m_minNextPollInterval = rval.m_minNextPollInterval;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlClientQueryResponse::Free(void)
{
    wctpXmlOperation::Free();

    m_vClientMessage.destroy();
    if(m_pNoMessages != NULL) 
    {
        delete m_pNoMessages;
        m_pNoMessages = NULL;
    }
    if(m_pFailure != NULL) 
    {
        delete m_pFailure;
        m_pFailure = NULL;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Clear object (reset object as if it were just constructed)

void
wctpXmlClientQueryResponse::Clear(void)
{
    Free();
    Init();
}


//////////////////////////////////////////////////////////////////////////////
// Construct an XML string for ClientMessage object(s)

XString
wctpXmlClientQueryResponse::AssembleClientMessage(XmlContext& ctx)
{
    XString dest;
    int i;
    for(i=0; i<m_vClientMessage.size(); i++) 
    {
        dest += m_vClientMessage[i]->AssembleOperation(ctx); 
    }
    return dest;
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlClientQueryResponse::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-ClientQueryResponse");
    dest += ctx.XmlAttribute(
        "minNextPollInterval",m_minNextPollInterval);
    dest += ctx.IndentText(">");
    if(m_ClientQueryResponseType == WCTP_XML_CLIENT_MESSAGE)
    {
        dest += AssembleClientMessage(ctx);
    }
    if(m_ClientQueryResponseType == WCTP_XML_NO_MESSAGES && 
       m_pNoMessages != NULL)
    {
        dest += m_pNoMessages->AssembleOperation(ctx);
    }
    if(m_ClientQueryResponseType == WCTP_XML_FAILURE && 
       m_pFailure != NULL)
    {
        dest += m_pFailure->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-ClientQueryResponse>");
    return dest;     
}

//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlClientQueryResponse::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "minNextPollInterval")
    {
        m_minNextPollInterval = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-ClientQueryResponse";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse one or more ClientMessage object(s)

wctp_parse_t
wctpXmlClientQueryResponse::ParseClientMessage(
    XString& msg,
    XmlNode *pTree,
    int& pos)
{
    bool flag = false;
    while(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        if(pChild->element != "wctp-ClientMessage")
        {
            break;
        }
        flag = true;
        pos++;
        wctpXmlClientMessage *pClientMessage = AddClientMessage();
        wctp_parse_t res = pClientMessage->ParseObject(msg, pChild); 
        if(res != WCTP_PARSE_OKAY)
        {
            return res; 
        }
    }
    if(!flag) 
    {
        msg = "expect at least one wctp-ClientMessage ";
        msg += "inside wctp-ClientQueryResponse";
        return WCTP_PARSE_ERROR;        
    }
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Parse switch for choice among set of objects (must match one)

wctp_parse_t 
wctpXmlClientQueryResponse::ParseObjectSwitch(
    XString& msg,
    XmlNode *pTree, 
    int& pos)
{
    wctp_parse_t res = WCTP_PARSE_ERROR;
    if(pos >= pTree->children.size())
    {
        msg = "An element is required inside wctp-ClientQueryResponse";
        return res;
    }
    XmlNode *pChild = pTree->children[pos];
    if(pChild->element == "wctp-ClientMessage")
    {
        HoldClientMessage();
        res = ParseClientMessage(msg,pTree,pos);
    }
    else if(pChild->element == "wctp-NoMessages")
    {
        HoldNoMessages();
        pos++;
        res = m_pNoMessages->ParseObject(msg,pChild);
    }
    else if(pChild->element == "wctp-Failure")
    {
        HoldFailure();
        pos++;
        res = m_pFailure->ParseObject(msg,pChild);
    }
    else 
    {
        msg = "unexpected element |";
        msg += pChild->element;
        msg += "| found inside wctp-ClientQueryResponse";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlClientQueryResponse::ParseObject(XString& msg,XmlNode *pTree)
{
    
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-ClientQueryResponse")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-ClientQueryResponse, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-ClientQueryResponse does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    wctp_parse_t res;

    //
    // Load in an object from a set
    //
    int pos = 0;

    res = ParseObjectSwitch(msg, pTree, pos);
    if(res != WCTP_PARSE_OKAY)
    { 
        return res; 
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-ClientQueryResponse element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Accessor to find which object type is contained

wctp_xml_t 
wctpXmlClientQueryResponse::GetClientQueryResponseType(void)
{
    return m_ClientQueryResponseType;
}

//////////////////////////////////////////////////////////////////////////////
// Clear all ClientMessage elements

void
wctpXmlClientQueryResponse::ClearClientMessage(void) 
{
    m_vClientMessage.destroy();
}

//////////////////////////////////////////////////////////////////////////////
// Get ClientMessage element at given position

wctpXmlClientMessage *
wctpXmlClientQueryResponse::GetClientMessage(int pos)
{
    if(pos >= 0 && pos < m_vClientMessage.size())
    {
        return m_vClientMessage[pos];
    }
    return NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Add ClientMessage element to end of array

wctpXmlClientMessage *
wctpXmlClientQueryResponse::AddClientMessage(void)
{
    wctpXmlClientMessage *pClientMessage = 
        new wctpXmlClientMessage;
    m_vClientMessage.push(pClientMessage);
    return pClientMessage;
}

//////////////////////////////////////////////////////////////////////////////
// Erase ClientMessage element at given position

void
wctpXmlClientQueryResponse::EraseClientMessage(int pos)
{
    wctpXmlClientMessage *pClientMessage = 
        m_vClientMessage.remove(pos);
    delete pClientMessage;
    pClientMessage = NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Return number of ClientMessage elements

int
wctpXmlClientQueryResponse::NumClientMessage(void)
{
    return m_vClientMessage.size();
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold ClientMessage object 

void
wctpXmlClientQueryResponse::HoldClientMessage(void)
{
    m_ClientQueryResponseType = WCTP_XML_CLIENT_MESSAGE;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to NoMessages object
// This function will never return NULL unless new fails...

wctpXmlNoMessages*
wctpXmlClientQueryResponse::GetNoMessages(void) 
{
    if(m_pNoMessages == NULL)
    {
        m_pNoMessages = new wctpXmlNoMessages;
    }
    return m_pNoMessages;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold NoMessages object 

void
wctpXmlClientQueryResponse::HoldNoMessages(void) 
{
    m_ClientQueryResponseType = WCTP_XML_NO_MESSAGES;
    if(m_pNoMessages == NULL)
    {
        m_pNoMessages = new wctpXmlNoMessages;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Failure object
// This function will never return NULL unless new fails...

wctpXmlFailure*
wctpXmlClientQueryResponse::GetFailure(void) 
{
    if(m_pFailure == NULL)
    {
        m_pFailure = new wctpXmlFailure;
    }
    return m_pFailure;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold Failure object 

void
wctpXmlClientQueryResponse::HoldFailure(void) 
{
    m_ClientQueryResponseType = WCTP_XML_FAILURE;
    if(m_pFailure == NULL)
    {
        m_pFailure = new wctpXmlFailure;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_minNextPollInterval attribute

const char * 
wctpXmlClientQueryResponse::GetMinNextPollInterval(void) const 
{
    return m_minNextPollInterval;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_minNextPollInterval attribute

void  
wctpXmlClientQueryResponse::SetMinNextPollInterval(const char * newValue)
{
    m_minNextPollInterval = newValue;
}

