/* File:  wctpXmlFailure.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlFailure.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlFailure::wctpXmlFailure()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlFailure::wctpXmlFailure(
    const wctpXmlFailure &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlFailure::~wctpXmlFailure()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlFailure&
wctpXmlFailure::operator=(
    const wctpXmlFailure &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlFailure::Init(void)
{
    wctpXmlOperation::Init();

    m_errorCode = "";
    m_errorText = "";
    m_pcdata = "";
    
    m_title = FAILURE_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;
    
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlFailure::Copy(const wctpXmlFailure &rval)
{
    //
    // Copy member variables
    //

    wctpXmlOperation::Copy(rval);

    m_errorCode = rval.m_errorCode;
    m_errorText = rval.m_errorText;
    m_pcdata = rval.m_pcdata;
    
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlFailure::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    XString temp;

    temp = "<";
    temp += m_xmlOperation;

    dest += ctx.OpenIndent(temp);
    dest += ctx.XmlAttribute("errorCode",m_errorCode);
    dest += ctx.XmlAttribute("errorText",m_errorText);
    dest += ctx.IndentText(">");
    dest += ctx.XmlPcdata(m_pcdata);

    temp = "</";
    temp += m_xmlOperation;
    temp += ">";
    dest += ctx.CloseIndent(temp);

    return dest;     
}

//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlFailure::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "errorCode")
    {
        m_errorCode = val;
    }
    else if(key == "errorText")
    {
        m_errorText = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element ";
        msg += m_xmlOperation;
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 
wctp_parse_t 
wctpXmlFailure::ParseObject(XString& msg,XmlNode *pTree)
{
    //
    // make sure the element matches me
    //
    if (pTree->element != m_xmlOperation)
    {
        msg = "element mismatch, expected ";
        msg += m_xmlOperation;
        msg += ", but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }

    //
    // load in attribute values
    //
    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

    //
    // collect pcdata 
    //

    m_pcdata = pTree->pcdata.both_trim();

    //
    // Should not be any child objects
    //

    if(pTree->children.size() > 0)
    {
        msg = m_xmlOperation;
        msg += " element does not accept any child elements";
        return WCTP_PARSE_ERROR;
    }
    
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_errorCode attribute

const char * 
wctpXmlFailure::GetErrorCode(void) const 
{
    return m_errorCode;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_errorCode attribute

void  
wctpXmlFailure::SetErrorCode(const char * newValue)
{
    m_errorCode = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_errorText attribute

const char * 
wctpXmlFailure::GetErrorText(void) const 
{
    return m_errorText;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_errorText attribute

void  
wctpXmlFailure::SetErrorText(const char * newValue)
{
    m_errorText = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_pcdata attribute

const char * 
wctpXmlFailure::GetPcdata(void) const 
{
    return m_pcdata;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_pcdata attribute

void  
wctpXmlFailure::SetPcdata(const char * newValue)
{
    m_pcdata = newValue;
}



