/* File:  wctpXmlLookupData.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlLookupData.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlLookupData::wctpXmlLookupData()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlLookupData::wctpXmlLookupData(
    const wctpXmlLookupData &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlLookupData::~wctpXmlLookupData()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlLookupData&
wctpXmlLookupData::operator=(
    const wctpXmlLookupData &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlLookupData::Init(void)
{
    wctpXmlOperation::Init();

    m_canRespond = WCTP_BOOL_DEFAULT;
    m_maxMessageLength = "";
    m_mcrSupported = WCTP_BOOL_DEFAULT;
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlLookupData::Copy(const wctpXmlLookupData &rval)
{
    //
    // Copy member variables
    //

    wctpXmlOperation::Copy(rval);

    m_canRespond = rval.m_canRespond;
    m_maxMessageLength = rval.m_maxMessageLength;
    m_mcrSupported = rval.m_mcrSupported;
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlLookupData::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-LookupData");
    dest += ctx.XmlAttribute(
        "canRespond", wctpXml::bool_to_str(m_canRespond));
    dest += ctx.XmlAttribute(
        "maxMessageLength",m_maxMessageLength);
    dest += ctx.XmlAttribute(
        "mcrSupported", wctpXml::bool_to_str(m_mcrSupported));
    dest += ctx.CloseIndent("/>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlLookupData::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "canRespond")
    {
        m_canRespond = wctpXml::str_to_bool(val.c_str());
    }
    else if(key == "maxMessageLength")
    {
        m_maxMessageLength = val;
    }
    else if(key == "mcrSupported")
    {
        m_mcrSupported = wctpXml::str_to_bool(val.c_str());
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-LookupData";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlLookupData::ParseObject(XString& msg, XmlNode *pTree)
{
    

    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-LookupData")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-LookupData, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-LookupData does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Should not be any child objects
    //

    if(pTree->children.size() > 0)
    {
        msg = "wctp-LookupData element does not accept any child elements";
        return WCTP_PARSE_ERROR;
    }
    
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_canRespond attribute

wctp_bool_t 
wctpXmlLookupData::GetCanRespond(void) const 
{
    return m_canRespond;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_canRespond attribute

void  
wctpXmlLookupData::SetCanRespond(wctp_bool_t newValue)
{
    m_canRespond = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_maxMessageLength attribute

const char * 
wctpXmlLookupData::GetMaxMessageLength(void) const 
{
    return m_maxMessageLength;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_maxMessageLength attribute

void  
wctpXmlLookupData::SetMaxMessageLength(const char * newValue)
{
    m_maxMessageLength = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_mcrSupported attribute

wctp_bool_t 
wctpXmlLookupData::GetMcrSupported(void) const 
{
    return m_mcrSupported;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_mcrSupported attribute

void  
wctpXmlLookupData::SetMcrSupported(wctp_bool_t newValue)
{
    m_mcrSupported = newValue;
}

