/* File:  wctpXmlLookupMessageControl.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlLookupMessageControl.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlLookupMessageControl::wctpXmlLookupMessageControl()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlLookupMessageControl::wctpXmlLookupMessageControl(
    const wctpXmlLookupMessageControl &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlLookupMessageControl::~wctpXmlLookupMessageControl()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlLookupMessageControl&
wctpXmlLookupMessageControl::operator=(
    const wctpXmlLookupMessageControl &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlLookupMessageControl::Init(void)
{
    wctpXmlOperation::Init();

    m_messageID = "";
    m_sendResponsesToID = "";
    m_transactionID = "";
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlLookupMessageControl::Copy(const wctpXmlLookupMessageControl &rval)
{
    //
    // Copy member variables
    //

    wctpXmlOperation::Copy(rval);

    m_messageID = rval.m_messageID;
    m_sendResponsesToID = rval.m_sendResponsesToID;
    m_transactionID = rval.m_transactionID;
}




//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlLookupMessageControl::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-LookupMessageControl");
    dest += ctx.XmlAttribute(
        "messageID",m_messageID);
    dest += ctx.XmlAttribute(
        "sendResponsesToID",m_sendResponsesToID);
    dest += ctx.XmlAttribute(
        "transactionID",m_transactionID);
    dest += ctx.CloseIndent("/>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlLookupMessageControl::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "messageID")
    {
        m_messageID = val;
    }
    else if(key == "sendResponsesToID")
    {
        m_sendResponsesToID = val;
    }
    else if(key == "transactionID")
    {
        m_transactionID = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-LookupMessageControl";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlLookupMessageControl::ParseObject(XString& msg, XmlNode *pTree)
{
  

    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-LookupMessageControl")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-LookupMessageControl, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-LookupMessageControl does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Should not be any child objects
    //

    if(pTree->children.size() > 0)
    {
        msg = "wctp-LookupMessageControl element does not accept any child elements";
        return WCTP_PARSE_ERROR;
    }
    
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_messageID attribute

const char * 
wctpXmlLookupMessageControl::GetMessageID(void) const 
{
    return m_messageID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_messageID attribute

void  
wctpXmlLookupMessageControl::SetMessageID(const char * newValue)
{
    m_messageID = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_sendResponsesToID attribute

const char * 
wctpXmlLookupMessageControl::GetSendResponsesToID(void) const 
{
    return m_sendResponsesToID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_sendResponsesToID attribute

void  
wctpXmlLookupMessageControl::SetSendResponsesToID(const char * newValue)
{
    m_sendResponsesToID = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_transactionID attribute

const char * 
wctpXmlLookupMessageControl::GetTransactionID(void) const 
{
    return m_transactionID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_transactionID attribute

void  
wctpXmlLookupMessageControl::SetTransactionID(const char * newValue)
{
    m_transactionID = newValue;
}

