/* File:  wctpXmlLookupResponse.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include <stdio.h>

#include "wctpXmlLookupResponse.hpp"

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlLookupResponse::wctpXmlLookupResponse()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlLookupResponse::wctpXmlLookupResponse(
    const wctpXmlLookupResponse &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlLookupResponse::~wctpXmlLookupResponse()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlLookupResponse&
wctpXmlLookupResponse::operator=(
    const wctpXmlLookupResponse &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlLookupResponse::Init(void)
{
    wctpXmlOperation::Init();
    
    m_OperationType = WCTP_XML_LOOKUP_RESPONSE;


    m_LookupResponseType = WCTP_XML_UNDEFINED;
    m_pOriginator = new wctpXmlOriginator;
    m_pRecipient = new wctpXmlRecipient;
    m_pLookupData = NULL;
    m_pFailure = NULL;
    m_responseToMessageID = "";
    m_transactionID = "";

    m_title = LOOKUP_RESPONSE_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlLookupResponse::Copy(const wctpXmlLookupResponse &rval)
{
    wctpXmlOperation::Copy(rval);

    //
    // Copy object type
    //

    m_LookupResponseType = rval.m_LookupResponseType;

    //
    // Copy pOriginator object if available (should always hold one)
    //

    if(rval.m_pOriginator != NULL)
    {
        m_pOriginator = new wctpXmlOriginator(
            *rval.m_pOriginator);
    }
    else
    {
        m_pOriginator = new wctpXmlOriginator;
    }

    //
    // Copy pRecipient object if available (should always hold one)
    //

    if(rval.m_pRecipient != NULL)
    {
        m_pRecipient = new wctpXmlRecipient(
            *rval.m_pRecipient);
    }
    else
    {
        m_pRecipient = new wctpXmlRecipient;
    }

    //
    // Copy pLookupData object if available (maybe holds)
    //

    if(rval.m_pLookupData != NULL)
    {
        m_pLookupData = new wctpXmlLookupData(
            *rval.m_pLookupData);
    }
    else
    {
        m_pLookupData = NULL;
    }

    //
    // Copy pFailure object if available (maybe holds)
    //

    if(rval.m_pFailure != NULL)
    {
        m_pFailure = new wctpXmlFailure(
            *rval.m_pFailure);
    }
    else
    {
        m_pFailure = NULL;
    }

    //
    // Copy member variables
    //

    m_responseToMessageID = rval.m_responseToMessageID;
    m_transactionID = rval.m_transactionID;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlLookupResponse::Free(void)
{
    wctpXmlOperation::Free();

    if(m_pOriginator != NULL) 
    {
        delete m_pOriginator;
        m_pOriginator = NULL;
    }
    if(m_pRecipient != NULL) 
    {
        delete m_pRecipient;
        m_pRecipient = NULL;
    }
    if(m_pLookupData != NULL) 
    {
        delete m_pLookupData;
        m_pLookupData = NULL;
    }
    if(m_pFailure != NULL) 
    {
        delete m_pFailure;
        m_pFailure = NULL;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Clear object (reset object as if it were just constructed)

void
wctpXmlLookupResponse::Clear(void)
{
    Free();
    Init();
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlLookupResponse::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-LookupResponse");
    dest += ctx.XmlAttribute(
        "responseToMessageID",m_responseToMessageID);
    dest += ctx.XmlAttribute(
        "transactionID",m_transactionID);
    dest += ctx.IndentText(">");
    if(m_pOriginator != NULL)
    {
        dest += m_pOriginator->AssembleOperation(ctx);
    }
    if(m_pRecipient != NULL)
    {
        dest += m_pRecipient->AssembleOperation(ctx);
    }
    if(m_LookupResponseType == WCTP_XML_LOOKUP_DATA && 
       m_pLookupData != NULL)
    {
        dest += m_pLookupData->AssembleOperation(ctx);
    }
    if(m_LookupResponseType == WCTP_XML_FAILURE && 
       m_pFailure != NULL)
    {
        dest += m_pFailure->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-LookupResponse>");
    return dest;     
}

//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlLookupResponse::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "responseToMessageID")
    {
        m_responseToMessageID = val;
    }
    else if(key == "transactionID")
    {
        m_transactionID = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-LookupResponse";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse switch for choice among set of objects (must match one)

wctp_parse_t 
wctpXmlLookupResponse::ParseObjectSwitch(
    XString& msg,
    XmlNode *pTree, 
    int& pos)
{
    wctp_parse_t res = WCTP_PARSE_ERROR;
    if(pos >= pTree->children.size())
    {
        msg = "An element is required inside wctp-LookupResponse";
        return res;
    }
    XmlNode *pChild = pTree->children[pos];
    if(pChild->element == "wctp-LookupData")
    {
        HoldLookupData();
        pos++;
        res = m_pLookupData->ParseObject(msg,pChild);
    }
    else if(pChild->element == "wctp-Failure")
    {
        HoldFailure();
        pos++;
        res = m_pFailure->ParseObject(msg,pChild);
    }
    else 
    {
        msg = "unexpected element |";
        msg += pChild->element;
        msg += "| found inside wctp-LookupResponse";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlLookupResponse::ParseObject(XString& msg,XmlNode *pTree)
{
    int pos = 0;
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-LookupResponse")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-LookupResponse, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-LookupResponse does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    wctp_parse_t res;

    //
    // Load in wctp-Originator object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pOriginator->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Originator element expected in ";
        msg += "wctp-LookupResponse element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in wctp-Recipient object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pRecipient->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Recipient element expected in ";
        msg += "wctp-LookupResponse element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in an object from a set
    //

    res = ParseObjectSwitch(msg, pTree, pos);
    if(res != WCTP_PARSE_OKAY)
    { 
        return res; 
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-LookupResponse element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Accessor to find which object type is contained

wctp_xml_t 
wctpXmlLookupResponse::GetLookupResponseType(void)
{
    return m_LookupResponseType;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Originator object
// This function will never return NULL unless new fails...

wctpXmlOriginator*
wctpXmlLookupResponse::GetOriginator(void) 
{
    if(m_pOriginator == NULL)
    {
        m_pOriginator = new wctpXmlOriginator;
    }
    return m_pOriginator;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Recipient object
// This function will never return NULL unless new fails...

wctpXmlRecipient*
wctpXmlLookupResponse::GetRecipient(void) 
{
    if(m_pRecipient == NULL)
    {
        m_pRecipient = new wctpXmlRecipient;
    }
    return m_pRecipient;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to LookupData object
// This function will never return NULL unless new fails...

wctpXmlLookupData*
wctpXmlLookupResponse::GetLookupData(void) 
{
    if(m_pLookupData == NULL)
    {
        m_pLookupData = new wctpXmlLookupData;
    }
    return m_pLookupData;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold LookupData object 

void
wctpXmlLookupResponse::HoldLookupData(void) 
{
    m_LookupResponseType = WCTP_XML_LOOKUP_DATA;
    if(m_pLookupData == NULL)
    {
        m_pLookupData = new wctpXmlLookupData;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Failure object
// This function will never return NULL unless new fails...

wctpXmlFailure*
wctpXmlLookupResponse::GetFailure(void) 
{
    if(m_pFailure == NULL)
    {
        m_pFailure = new wctpXmlFailure;
    }
    return m_pFailure;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold Failure object 

void
wctpXmlLookupResponse::HoldFailure(void) 
{
    m_LookupResponseType = WCTP_XML_FAILURE;
    if(m_pFailure == NULL)
    {
        m_pFailure = new wctpXmlFailure;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_responseToMessageID attribute

const char * 
wctpXmlLookupResponse::GetResponseToMessageID(void) const 
{
    return m_responseToMessageID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_responseToMessageID attribute

void  
wctpXmlLookupResponse::SetResponseToMessageID(const char * newValue)
{
    m_responseToMessageID = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_transactionID attribute

const char * 
wctpXmlLookupResponse::GetTransactionID(void) const 
{
    return m_transactionID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_transactionID attribute

void  
wctpXmlLookupResponse::SetTransactionID(const char * newValue)
{
    m_transactionID = newValue;
}

