/* File:  wctpXmlLookupSubscriber.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlLookupSubscriber.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlLookupSubscriber::wctpXmlLookupSubscriber()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlLookupSubscriber::wctpXmlLookupSubscriber(
    const wctpXmlLookupSubscriber &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlLookupSubscriber::~wctpXmlLookupSubscriber()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlLookupSubscriber&
wctpXmlLookupSubscriber::operator=(
    const wctpXmlLookupSubscriber &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlLookupSubscriber::Init(void)
{
    wctpXmlOperation::Init();

    m_OperationType = WCTP_XML_LOOKUP_SUBSCRIBER;

    m_pOriginator = new wctpXmlOriginator;
    m_pLookupMessageControl = new wctpXmlLookupMessageControl;
    m_pRecipient = new wctpXmlRecipient;
        

    m_title = LOOKUP_SUBSCRIBER_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlLookupSubscriber::Copy(const wctpXmlLookupSubscriber &rval)
{
    
    wctpXmlOperation::Copy(rval);
        
    //
    // Copy pOriginator object if available (should always hold one)
    //

    if(rval.m_pOriginator != NULL)
    {
        m_pOriginator = new wctpXmlOriginator(
            *rval.m_pOriginator);
    }
    else
    {
        m_pOriginator = new wctpXmlOriginator;
    }

    //
    // Copy pLookupMessageControl object if available (should always hold one)
    //

    if(rval.m_pLookupMessageControl != NULL)
    {
        m_pLookupMessageControl = new wctpXmlLookupMessageControl(
            *rval.m_pLookupMessageControl);
    }
    else
    {
        m_pLookupMessageControl = new wctpXmlLookupMessageControl;
    }

    //
    // Copy pRecipient object if available (should always hold one)
    //

    if(rval.m_pRecipient != NULL)
    {
        m_pRecipient = new wctpXmlRecipient(
            *rval.m_pRecipient);
    }
    else
    {
        m_pRecipient = new wctpXmlRecipient;
    }

    m_xmlOperation = rval.m_xmlOperation;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlLookupSubscriber::Free(void)
{
    wctpXmlOperation::Free();

    if(m_pOriginator != NULL) 
    {
        delete m_pOriginator;
        m_pOriginator = NULL;
    }
    if(m_pLookupMessageControl != NULL) 
    {
        delete m_pLookupMessageControl;
        m_pLookupMessageControl = NULL;
    }
    if(m_pRecipient != NULL) 
    {
        delete m_pRecipient;
        m_pRecipient = NULL;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Clear object (reset object as if it were just constructed)

void
wctpXmlLookupSubscriber::Clear(void)
{
    Free();
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlLookupSubscriber::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    XString temp;

    temp = "<";
    temp += m_xmlOperation;
    temp += ">";
    dest += ctx.OpenIndent(temp);

    if(m_pOriginator != NULL)
    {
        dest += m_pOriginator->AssembleOperation(ctx);
    }
    if(m_pLookupMessageControl != NULL)
    {
        dest += m_pLookupMessageControl->AssembleOperation(ctx);
    }
    if(m_pRecipient != NULL)
    {
        dest += m_pRecipient->AssembleOperation(ctx);
    }

    temp =  "</";
    temp += m_xmlOperation;
    temp += ">";
    dest += ctx.CloseIndent(temp);

    return dest;     
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlLookupSubscriber::ParseObject(XString& msg,XmlNode *pTree)
{
    int pos = 0;
    //
    // make sure the element matches me
    //

    if(m_xmlOperation != pTree->element)
    {
        msg = "element mismatch, expected ";
        msg += m_xmlOperation;
        msg += ", but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }

    //
    // make sure no attributes where specified 
    //

    if(pTree->attributes.size() > 0)
    {
        msg = m_xmlOperation;
        msg += " does not have any attributes";
        return WCTP_PARSE_ERROR;
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = m_xmlOperation;
        msg += " does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

     wctp_parse_t res;

    //
    // Load in wctp-Originator object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pOriginator->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Originator element expected in ";
        msg += m_xmlOperation;
        msg += " element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in wctp-LookupMessageControl object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pLookupMessageControl->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-LookupMessageControl element expected in ";
        msg += m_xmlOperation;
        msg += " element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in wctp-Recipient object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pRecipient->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Recipient element expected in ";
        msg += m_xmlOperation;
        msg += " element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = m_xmlOperation;
        msg += " element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Originator object
// This function will never return NULL unless new fails...

wctpXmlOriginator*
wctpXmlLookupSubscriber::GetOriginator(void) 
{
    if(m_pOriginator == NULL)
    {
        m_pOriginator = new wctpXmlOriginator;
    }
    return m_pOriginator;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to LookupMessageControl object
// This function will never return NULL unless new fails...

wctpXmlLookupMessageControl*
wctpXmlLookupSubscriber::GetLookupMessageControl(void) 
{
    if(m_pLookupMessageControl == NULL)
    {
        m_pLookupMessageControl = new wctpXmlLookupMessageControl;
    }
    return m_pLookupMessageControl;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Recipient object
// This function will never return NULL unless new fails...

wctpXmlRecipient*
wctpXmlLookupSubscriber::GetRecipient(void) 
{
    if(m_pRecipient == NULL)
    {
        m_pRecipient = new wctpXmlRecipient;
    }
    return m_pRecipient;
}

// Sets value of XML strin g
void 
wctpXmlLookupSubscriber::SetXmlString(const char* newValue)
{
    m_xmlOperation = newValue;
}

