/* File:  wctpXmlMessageControl.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlMessageControl.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlMessageControl::wctpXmlMessageControl()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlMessageControl::wctpXmlMessageControl(
    const wctpXmlMessageControl &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlMessageControl::~wctpXmlMessageControl()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlMessageControl&
wctpXmlMessageControl::operator=(
    const wctpXmlMessageControl &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlMessageControl::Init(void)
{
    wctpXmlOperation::Init();

    m_allowResponse = WCTP_BOOL_TRUE;
    m_allowTruncation = WCTP_BOOL_DEFAULT;
    m_deliveryAfter = "";
    m_deliveryBefore = "";
    m_deliveryPriority = WCTP_PRIORITY_DEFAULT;
    m_messageID = "";
    m_notifyWhenDelivered = WCTP_BOOL_DEFAULT;
    m_notifyWhenQueued = WCTP_BOOL_DEFAULT;
    m_notifyWhenRead = WCTP_BOOL_DEFAULT;
    m_preformatted = WCTP_BOOL_DEFAULT;
    m_sendResponsesToID = "";
    m_transactionID = "";
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlMessageControl::Copy(const wctpXmlMessageControl &rval)
{
    //
    // Copy member variables
    //
    wctpXmlOperation::Copy(rval);

    m_allowResponse = rval.m_allowResponse;
    m_allowTruncation = rval.m_allowTruncation;
    m_deliveryAfter = rval.m_deliveryAfter;
    m_deliveryBefore = rval.m_deliveryBefore;
    m_deliveryPriority = rval.m_deliveryPriority;
    m_messageID = rval.m_messageID;
    m_notifyWhenDelivered = rval.m_notifyWhenDelivered;
    m_notifyWhenQueued = rval.m_notifyWhenQueued;
    m_notifyWhenRead = rval.m_notifyWhenRead;
    m_preformatted = rval.m_preformatted;
    m_sendResponsesToID = rval.m_sendResponsesToID;
    m_transactionID = rval.m_transactionID;
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlMessageControl::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-MessageControl");
    dest += ctx.XmlAttribute(
        "allowResponse", wctpXml::bool_to_str(m_allowResponse));
    dest += ctx.XmlAttribute(
        "allowTruncation", wctpXml::bool_to_str(m_allowTruncation));
    dest += ctx.XmlAttribute(
        "deliveryAfter",m_deliveryAfter);
    dest += ctx.XmlAttribute(
        "deliveryBefore",m_deliveryBefore);
    dest += ctx.XmlAttribute(
        "deliveryPriority", wctpXml::priority_to_str(m_deliveryPriority));
    dest += ctx.XmlAttribute(
        "messageID",m_messageID);
    dest += ctx.XmlAttribute(
        "notifyWhenDelivered", wctpXml::bool_to_str(m_notifyWhenDelivered));
    dest += ctx.XmlAttribute(
        "notifyWhenQueued", wctpXml::bool_to_str(m_notifyWhenQueued));
    dest += ctx.XmlAttribute(
        "notifyWhenRead", wctpXml::bool_to_str(m_notifyWhenRead));
    dest += ctx.XmlAttribute(
        "preformatted", wctpXml::bool_to_str(m_preformatted));
    dest += ctx.XmlAttribute(
        "sendResponsesToID",m_sendResponsesToID);
    dest += ctx.XmlAttribute(
        "transactionID",m_transactionID);
    dest += ctx.CloseIndent("/>");
    return dest;     
}

//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlMessageControl::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "allowResponse")
    {
        m_allowResponse = wctpXml::str_to_bool(val.c_str());
    }
    else if(key == "allowTruncation")
    {
        m_allowTruncation = wctpXml::str_to_bool(val.c_str());
    }
    else if(key == "deliveryAfter")
    {
        m_deliveryAfter = val;
    }
    else if(key == "deliveryBefore")
    {
        m_deliveryBefore = val;
    }
    else if(key == "deliveryPriority")
    {
        m_deliveryPriority = wctpXml::str_to_priority(val.c_str());
    }
    else if(key == "messageID")
    {
        m_messageID = val;
    }
    else if(key == "notifyWhenDelivered")
    {
        m_notifyWhenDelivered = wctpXml::str_to_bool(val.c_str());
    }
    else if(key == "notifyWhenQueued")
    {
        m_notifyWhenQueued = wctpXml::str_to_bool(val.c_str());
    }
    else if(key == "notifyWhenRead")
    {
        m_notifyWhenRead = wctpXml::str_to_bool(val.c_str());
    }
    else if(key == "preformatted")
    {
        m_preformatted = wctpXml::str_to_bool(val.c_str());
    }
    else if(key == "sendResponsesToID")
    {
        m_sendResponsesToID = val;
    }
    else if(key == "transactionID")
    {
        m_transactionID = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-MessageControl";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlMessageControl::ParseObject(XString& msg, XmlNode *pTree)
{

    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-MessageControl")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-MessageControl, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-MessageControl does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Should not be any child objects
    //

    if(pTree->children.size() > 0)
    {
        msg = "wctp-MessageControl element does not accept any child elements";
        return WCTP_PARSE_ERROR;
    }
    
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_allowResponse attribute

wctp_bool_t 
wctpXmlMessageControl::GetAllowResponse(void) const 
{
    return m_allowResponse;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_allowResponse attribute

void  
wctpXmlMessageControl::SetAllowResponse(wctp_bool_t newValue)
{
    m_allowResponse = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_allowTruncation attribute

wctp_bool_t 
wctpXmlMessageControl::GetAllowTruncation(void) const 
{
    return m_allowTruncation;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_allowTruncation attribute

void  
wctpXmlMessageControl::SetAllowTruncation(wctp_bool_t newValue)
{
    m_allowTruncation = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_deliveryAfter attribute

const char * 
wctpXmlMessageControl::GetDeliveryAfter(void) const 
{
    return m_deliveryAfter;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_deliveryAfter attribute

void  
wctpXmlMessageControl::SetDeliveryAfter(const char * newValue)
{
    m_deliveryAfter = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_deliveryBefore attribute

const char * 
wctpXmlMessageControl::GetDeliveryBefore(void) const 
{
    return m_deliveryBefore;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_deliveryBefore attribute

void  
wctpXmlMessageControl::SetDeliveryBefore(const char * newValue)
{
    m_deliveryBefore = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_deliveryPriority attribute

wctp_priority_t 
wctpXmlMessageControl::GetDeliveryPriority(void) const 
{
    return m_deliveryPriority;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_deliveryPriority attribute

void  
wctpXmlMessageControl::SetDeliveryPriority(wctp_priority_t newValue)
{
    m_deliveryPriority = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_messageID attribute

const char * 
wctpXmlMessageControl::GetMessageID(void) const 
{
    return m_messageID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_messageID attribute

void  
wctpXmlMessageControl::SetMessageID(const char * newValue)
{
    m_messageID = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_notifyWhenDelivered attribute

wctp_bool_t 
wctpXmlMessageControl::GetNotifyWhenDelivered(void) const 
{
    return m_notifyWhenDelivered;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_notifyWhenDelivered attribute

void  
wctpXmlMessageControl::SetNotifyWhenDelivered(wctp_bool_t newValue)
{
    m_notifyWhenDelivered = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_notifyWhenQueued attribute

wctp_bool_t 
wctpXmlMessageControl::GetNotifyWhenQueued(void) const 
{
    return m_notifyWhenQueued;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_notifyWhenQueued attribute

void  
wctpXmlMessageControl::SetNotifyWhenQueued(wctp_bool_t newValue)
{
    m_notifyWhenQueued = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_notifyWhenRead attribute

wctp_bool_t 
wctpXmlMessageControl::GetNotifyWhenRead(void) const 
{
    return m_notifyWhenRead;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_notifyWhenRead attribute

void  
wctpXmlMessageControl::SetNotifyWhenRead(wctp_bool_t newValue)
{
    m_notifyWhenRead = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_preformatted attribute

wctp_bool_t 
wctpXmlMessageControl::GetPreformatted(void) const 
{
    return m_preformatted;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_preformatted attribute

void  
wctpXmlMessageControl::SetPreformatted(wctp_bool_t newValue)
{
    m_preformatted = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_sendResponsesToID attribute

const char * 
wctpXmlMessageControl::GetSendResponsesToID(void) const 
{
    return m_sendResponsesToID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_sendResponsesToID attribute

void  
wctpXmlMessageControl::SetSendResponsesToID(const char * newValue)
{
    m_sendResponsesToID = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_transactionID attribute

const char * 
wctpXmlMessageControl::GetTransactionID(void) const 
{
    return m_transactionID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_transactionID attribute

void  
wctpXmlMessageControl::SetTransactionID(const char * newValue)
{
    m_transactionID = newValue;
}

