/* File:  wctpXmlMessageReceived.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlMessageReceived.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlMessageReceived::wctpXmlMessageReceived()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlMessageReceived::wctpXmlMessageReceived(
    const wctpXmlMessageReceived &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlMessageReceived::~wctpXmlMessageReceived()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlMessageReceived&
wctpXmlMessageReceived::operator=(
    const wctpXmlMessageReceived &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlMessageReceived::Init(void)
{
    wctpXmlOperation::Init();

    m_MessageReceivedType = WCTP_XML_UNDEFINED;
    m_pFailure = NULL;
    m_pSuccess = NULL;
    m_sequenceNo = "";

    m_title = MESSAGE_RECEIVED_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlMessageReceived::Copy(const wctpXmlMessageReceived &rval)
{
    //
    // Copy object type
    //

    wctpXmlOperation::Copy(rval);

    m_MessageReceivedType = rval.m_MessageReceivedType;

    //
    // Copy pFailure object if available (maybe holds)
    //

    if(rval.m_pFailure != NULL)
    {
        m_pFailure = new wctpXmlFailure(
            *rval.m_pFailure);
    }
    else
    {
        m_pFailure = NULL;
    }

    //
    // Copy pSuccess object if available (maybe holds)
    //

    if(rval.m_pSuccess != NULL)
    {
        m_pSuccess = new wctpXmlSuccess(
            *rval.m_pSuccess);
    }
    else
    {
        m_pSuccess = NULL;
    }

    //
    // Copy member variables
    //

    m_sequenceNo = rval.m_sequenceNo;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlMessageReceived::Free(void)
{
    wctpXmlOperation::Free();

    if(m_pFailure != NULL) 
    {
        delete m_pFailure;
        m_pFailure = NULL;
    }
    if(m_pSuccess != NULL) 
    {
        delete m_pSuccess;
        m_pSuccess = NULL;
    }
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlMessageReceived::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-MessageReceived");
    dest += ctx.XmlAttribute(
        "sequenceNo",m_sequenceNo);
    dest += ctx.IndentText(">");
    if(m_MessageReceivedType == WCTP_XML_FAILURE && 
       m_pFailure != NULL)
    {
        dest += m_pFailure->AssembleOperation(ctx);
    }
    if(m_MessageReceivedType == WCTP_XML_SUCCESS && 
       m_pSuccess != NULL)
    {
        dest += m_pSuccess->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-MessageReceived>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlMessageReceived::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "sequenceNo")
    {
        m_sequenceNo = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-MessageReceived";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse switch for choice among set of objects (must match one)

wctp_parse_t 
wctpXmlMessageReceived::ParseObjectSwitch(
    XString& msg,
    XmlNode *pTree, 
    int& pos)
{
    wctp_parse_t res = WCTP_PARSE_ERROR;
    if(pos >= pTree->children.size())
    {
        msg = "An element is required inside wctp-MessageReceived";
        return res;
    }
    XmlNode *pChild = pTree->children[pos];
    if(pChild->element == "wctp-Failure")
    {
        HoldFailure();
        pos++;
        res = m_pFailure->ParseObject(msg,pChild);
    }
    else if(pChild->element == "wctp-Success")
    {
        HoldSuccess();
        pos++;
        res = m_pSuccess->ParseObject(msg,pChild);
    }
    else 
    {
        msg = "unexpected element |";
        msg += pChild->element;
        msg += "| found inside wctp-MessageReceived";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlMessageReceived::ParseObject(
    XString& msg,
    XmlNode *pTree)
{
   

    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-MessageReceived")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-MessageReceived, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-MessageReceived does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    int pos = 0;
    wctp_parse_t res;

    //
    // Load in an object from a set
    //

    res = ParseObjectSwitch(msg, pTree, pos);
    if(res != WCTP_PARSE_OKAY)
    { 
        return res; 
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-MessageReceived element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Accessor to find which object type is contained

wctp_xml_t 
wctpXmlMessageReceived::GetMessageReceivedType(void)
{
    return m_MessageReceivedType;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Failure object
// This function will never return NULL unless new fails...

wctpXmlFailure*
wctpXmlMessageReceived::GetFailure(void) 
{
    if(m_pFailure == NULL)
    {
        m_pFailure = new wctpXmlFailure;
    }
    return m_pFailure;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold Failure object 

void
wctpXmlMessageReceived::HoldFailure(void) 
{
    m_MessageReceivedType = WCTP_XML_FAILURE;
    if(m_pFailure == NULL)
    {
        m_pFailure = new wctpXmlFailure;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Success object
// This function will never return NULL unless new fails...

wctpXmlSuccess*
wctpXmlMessageReceived::GetSuccess(void) 
{
    if(m_pSuccess == NULL)
    {
        m_pSuccess = new wctpXmlSuccess;
    }
    return m_pSuccess;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold Success object 

void
wctpXmlMessageReceived::HoldSuccess(void) 
{
    m_MessageReceivedType = WCTP_XML_SUCCESS;
    if(m_pSuccess == NULL)
    {
        m_pSuccess = new wctpXmlSuccess;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_sequenceNo attribute

const char * 
wctpXmlMessageReceived::GetSequenceNo(void) const 
{
    return m_sequenceNo;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_sequenceNo attribute

void  
wctpXmlMessageReceived::SetSequenceNo(const char * newValue)
{
    m_sequenceNo = newValue;
}

