/* File:  wctpXmlNotification.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlNotification.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlNotification::wctpXmlNotification()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlNotification::wctpXmlNotification(
    const wctpXmlNotification &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlNotification::~wctpXmlNotification()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlNotification&
wctpXmlNotification::operator=(
    const wctpXmlNotification &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlNotification::Init(void)
{
    wctpXmlOperation::Init();

    m_type = WCTP_NOTIFICATION_DEFAULT;
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlNotification::Copy(const wctpXmlNotification &rval)
{
    //
    // Copy member variables
    //
    wctpXmlOperation::Copy(rval);

    m_type = rval.m_type;
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlNotification::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-Notification");
    dest += ctx.XmlAttribute(
        "type", wctpXml::notification_to_str(m_type));
    dest += ctx.CloseIndent("/>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlNotification::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "type")
    {
        m_type = wctpXml::str_to_notification(val.c_str());
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-Notification";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlNotification::ParseObject(XString& msg, XmlNode *pTree)
{
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-Notification")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-Notification, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-Notification does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Should not be any child objects
    //

    if(pTree->children.size() > 0)
    {
        msg = "wctp-Notification element does not accept any child elements";
        return WCTP_PARSE_ERROR;
    }
    
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_type attribute

wctp_notification_t 
wctpXmlNotification::GetType(void) const 
{
    return m_type;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_type attribute

void  
wctpXmlNotification::SetType(wctp_notification_t newValue)
{
    m_type = newValue;
}

