/* File:  wctpXmlPollForMessages.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlPollForMessages.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlPollForMessages::wctpXmlPollForMessages()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlPollForMessages::wctpXmlPollForMessages(
    const wctpXmlPollForMessages &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlPollForMessages::~wctpXmlPollForMessages()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlPollForMessages&
wctpXmlPollForMessages::operator=(
    const wctpXmlPollForMessages &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlPollForMessages::Init(void)
{
    wctpXmlOperation::Init();

    m_OperationType = WCTP_XML_POLL_FOR_MESSAGES;

    m_maxMessagesInBatch = "";
    m_pollerID = "";
    m_securityCode = "";

    m_title = POLL_FOR_MESSAGES_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;


}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlPollForMessages::Copy(const wctpXmlPollForMessages &rval)
{
    wctpXmlOperation::Copy(rval);
    
    //
    // Copy pMessageReceived object(s) if available (holds zero or more)
    //

    if(rval.m_vMessageReceived.size() > 0)
    {
        int i;
        for(i=0; i<rval.m_vMessageReceived.size(); i++)
        {
            wctpXmlMessageReceived *pMessageReceived = 
                new wctpXmlMessageReceived(
                *(rval.m_vMessageReceived[i]));
            m_vMessageReceived.push(pMessageReceived);
        }
    }

    //
    // Copy member variables
    //

    m_maxMessagesInBatch = rval.m_maxMessagesInBatch;
    m_pollerID = rval.m_pollerID;
    m_securityCode = rval.m_securityCode;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlPollForMessages::Free(void)
{
    wctpXmlOperation::Free();

    m_vMessageReceived.destroy();
}

//////////////////////////////////////////////////////////////////////////////
// Clear object (reset object as if it were just constructed)

void
wctpXmlPollForMessages::Clear(void)
{
    Free();
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Construct an XML string for MessageReceived object(s)

XString
wctpXmlPollForMessages::AssembleMessageReceived(XmlContext& ctx)
{
    XString dest;
    int i;
    for(i=0; i<m_vMessageReceived.size(); i++) 
    {
        dest += m_vMessageReceived[i]->AssembleOperation(ctx); 
    }
    return dest;
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlPollForMessages::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-PollForMessages");
    dest += ctx.XmlAttribute(
        "maxMessagesInBatch",m_maxMessagesInBatch);
    dest += ctx.XmlAttribute(
        "pollerID",m_pollerID);
    dest += ctx.XmlAttribute(
        "securityCode",m_securityCode);
    dest += ctx.IndentText(">");
    dest += AssembleMessageReceived(ctx);
    dest += ctx.CloseIndent("</wctp-PollForMessages>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlPollForMessages::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "maxMessagesInBatch")
    {
        m_maxMessagesInBatch = val;
    }
    else if(key == "pollerID")
    {
        m_pollerID = val;
    }
    else if(key == "securityCode")
    {
        m_securityCode = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-PollForMessages";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse zero or more MessageReceived object(s)

wctp_parse_t
wctpXmlPollForMessages::ParseMessageReceived(
    XString& msg,
    XmlNode *pTree,
    int& pos)
{
    while(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        if(pChild->element != "wctp-MessageReceived")
        {
            break;
        }
        pos++;
        wctpXmlMessageReceived *pMessageReceived = AddMessageReceived();
        wctp_parse_t res = pMessageReceived->ParseObject(msg, pChild); 
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlPollForMessages::ParseObject(XString& msg,XmlNode *pTree)
{
    int pos = 0;
    
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-PollForMessages")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-PollForMessages, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-PollForMessages does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    wctp_parse_t res;

     //
     // Load in zero or more MessageReceived objects
     //

     res = ParseMessageReceived(msg,pTree,pos);
     if(res != WCTP_PARSE_OKAY)
     {
         return res;
     }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-PollForMessages element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Clear all MessageReceived elements

void
wctpXmlPollForMessages::ClearMessageReceived(void) 
{
    m_vMessageReceived.destroy();
}

//////////////////////////////////////////////////////////////////////////////
// Get MessageReceived element at given position

wctpXmlMessageReceived *
wctpXmlPollForMessages::GetMessageReceived(int pos)
{
    if(pos >= 0 && pos < m_vMessageReceived.size())
    {
        return m_vMessageReceived[pos];
    }
    return NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Add MessageReceived element to end of array

wctpXmlMessageReceived *
wctpXmlPollForMessages::AddMessageReceived(void)
{
    wctpXmlMessageReceived *pMessageReceived = 
        new wctpXmlMessageReceived;
    m_vMessageReceived.push(pMessageReceived);
    return pMessageReceived;
}

//////////////////////////////////////////////////////////////////////////////
// Erase MessageReceived element at given position

void
wctpXmlPollForMessages::EraseMessageReceived(int pos)
{
    wctpXmlMessageReceived *pMessageReceived = 
        m_vMessageReceived.remove(pos);
    delete pMessageReceived;
    pMessageReceived = NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Return number of MessageReceived elements

int
wctpXmlPollForMessages::NumMessageReceived(void)
{
    return m_vMessageReceived.size();
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_maxMessagesInBatch attribute

const char * 
wctpXmlPollForMessages::GetMaxMessagesInBatch(void) const 
{
    return m_maxMessagesInBatch;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_maxMessagesInBatch attribute

void  
wctpXmlPollForMessages::SetMaxMessagesInBatch(const char * newValue)
{
    m_maxMessagesInBatch = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_pollerID attribute

const char * 
wctpXmlPollForMessages::GetPollerID(void) const 
{
    return m_pollerID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_pollerID attribute

void  
wctpXmlPollForMessages::SetPollerID(const char * newValue)
{
    m_pollerID = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_securityCode attribute

const char * 
wctpXmlPollForMessages::GetSecurityCode(void) const 
{
    return m_securityCode;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_securityCode attribute

void  
wctpXmlPollForMessages::SetSecurityCode(const char * newValue)
{
    m_securityCode = newValue;
}
