/* File:  wctpXmlRecipient.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlRecipient.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlRecipient::wctpXmlRecipient()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlRecipient::wctpXmlRecipient(
    const wctpXmlRecipient &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlRecipient::~wctpXmlRecipient()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlRecipient&
wctpXmlRecipient::operator=(
    const wctpXmlRecipient &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlRecipient::Init(void)
{
    wctpXmlOperation::Init();

    m_authorizationCode = "";
    m_recipientID = "";
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlRecipient::Copy(const wctpXmlRecipient &rval)
{
    //
    // Copy member variables
    //
    wctpXmlOperation::Copy(rval);

    m_authorizationCode = rval.m_authorizationCode;
    m_recipientID = rval.m_recipientID;
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlRecipient::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-Recipient");
    dest += ctx.XmlAttribute(
        "authorizationCode",m_authorizationCode);
    dest += ctx.XmlAttribute(
        "recipientID",m_recipientID);
    dest += ctx.CloseIndent("/>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlRecipient::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "authorizationCode")
    {
        m_authorizationCode = val;
    }
    else if(key == "recipientID")
    {
        m_recipientID = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-Recipient";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlRecipient::ParseObject(XString& msg,XmlNode *pTree)
{
   
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-Recipient")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-Recipient, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-Recipient does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Should not be any child objects
    //

    if(pTree->children.size() > 0)
    {
        msg = "wctp-Recipient element does not accept any child elements";
        return WCTP_PARSE_ERROR;
    }
    
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_authorizationCode attribute

const char * 
wctpXmlRecipient::GetAuthorizationCode(void) const 
{
    return m_authorizationCode;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_authorizationCode attribute

void  
wctpXmlRecipient::SetAuthorizationCode(const char * newValue)
{
    m_authorizationCode = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_recipientID attribute

const char * 
wctpXmlRecipient::GetRecipientID(void) const 
{
    return m_recipientID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_recipientID attribute

void  
wctpXmlRecipient::SetRecipientID(const char * newValue)
{
    m_recipientID = newValue;
}

