/* File:  wctpXmlResponseHeader.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlResponseHeader.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlResponseHeader::wctpXmlResponseHeader()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlResponseHeader::wctpXmlResponseHeader(
    const wctpXmlResponseHeader &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlResponseHeader::~wctpXmlResponseHeader()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlResponseHeader&
wctpXmlResponseHeader::operator=(
    const wctpXmlResponseHeader &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlResponseHeader::Init(void)
{
    wctpXmlOperation::Init();

    m_pOriginator = new wctpXmlOriginator;
    m_pMessageControl = new wctpXmlMessageControl;
    m_pRecipient = new wctpXmlRecipient;
    m_onBehalfOfRecipientID = "";
    m_respondingToTimestamp = "";
    m_responseTimestamp = "";
    m_responseToMessageID = "";
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlResponseHeader::Copy(const wctpXmlResponseHeader &rval)
{
    wctpXmlOperation::Copy(rval);

    //
    // Copy pOriginator object if available (should always hold one)
    //

    if(rval.m_pOriginator != NULL)
    {
        m_pOriginator = new wctpXmlOriginator(
            *rval.m_pOriginator);
    }
    else
    {
        m_pOriginator = new wctpXmlOriginator;
    }

    //
    // Copy pMessageControl object if available (should always hold one)
    //

    if(rval.m_pMessageControl != NULL)
    {
        m_pMessageControl = new wctpXmlMessageControl(
            *rval.m_pMessageControl);
    }
    else
    {
        m_pMessageControl = new wctpXmlMessageControl;
    }

    //
    // Copy pRecipient object if available (should always hold one)
    //

    if(rval.m_pRecipient != NULL)
    {
        m_pRecipient = new wctpXmlRecipient(
            *rval.m_pRecipient);
    }
    else
    {
        m_pRecipient = new wctpXmlRecipient;
    }

    //
    // Copy member variables
    //

    m_onBehalfOfRecipientID = rval.m_onBehalfOfRecipientID;
    m_respondingToTimestamp = rval.m_respondingToTimestamp;
    m_responseTimestamp = rval.m_responseTimestamp;
    m_responseToMessageID = rval.m_responseToMessageID;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlResponseHeader::Free(void)
{
    wctpXmlOperation::Free();

    if(m_pOriginator != NULL) 
    {
        delete m_pOriginator;
        m_pOriginator = NULL;
    }
    if(m_pMessageControl != NULL) 
    {
        delete m_pMessageControl;
        m_pMessageControl = NULL;
    }
    if(m_pRecipient != NULL) 
    {
        delete m_pRecipient;
        m_pRecipient = NULL;
    }
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlResponseHeader::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-ResponseHeader");
    dest += ctx.XmlAttribute(
        "onBehalfOfRecipientID",m_onBehalfOfRecipientID);
    dest += ctx.XmlAttribute(
        "respondingToTimestamp",m_respondingToTimestamp);
    dest += ctx.XmlAttribute(
        "responseTimestamp",m_responseTimestamp);
    dest += ctx.XmlAttribute(
        "responseToMessageID",m_responseToMessageID);
    dest += ctx.IndentText(">");
    if(m_pOriginator != NULL)
    {
        dest += m_pOriginator->AssembleOperation(ctx);
    }
    if(m_pMessageControl != NULL)
    {
        dest += m_pMessageControl->AssembleOperation(ctx);
    }
    if(m_pRecipient != NULL)
    {
        dest += m_pRecipient->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-ResponseHeader>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlResponseHeader::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "onBehalfOfRecipientID")
    {
        m_onBehalfOfRecipientID = val;
    }
    else if(key == "respondingToTimestamp")
    {
        m_respondingToTimestamp = val;
    }
    else if(key == "responseTimestamp")
    {
        m_responseTimestamp = val;
    }
    else if(key == "responseToMessageID")
    {
        m_responseToMessageID = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-ResponseHeader";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlResponseHeader::ParseObject(XString& msg, XmlNode *pTree)
{
    
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-ResponseHeader")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-ResponseHeader, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-ResponseHeader does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    int pos = 0;
    wctp_parse_t res;

    //
    // Load in wctp-Originator object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pOriginator->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Originator element expected in ";
        msg += "wctp-ResponseHeader element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in wctp-MessageControl object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pMessageControl->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-MessageControl element expected in ";
        msg += "wctp-ResponseHeader element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in wctp-Recipient object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pRecipient->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Recipient element expected in ";
        msg += "wctp-ResponseHeader element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-ResponseHeader element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Originator object
// This function will never return NULL unless new fails...

wctpXmlOriginator*
wctpXmlResponseHeader::GetOriginator(void) 
{
    if(m_pOriginator == NULL)
    {
        m_pOriginator = new wctpXmlOriginator;
    }
    return m_pOriginator;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to MessageControl object
// This function will never return NULL unless new fails...

wctpXmlMessageControl*
wctpXmlResponseHeader::GetMessageControl(void) 
{
    if(m_pMessageControl == NULL)
    {
        m_pMessageControl = new wctpXmlMessageControl;
    }
    return m_pMessageControl;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Recipient object
// This function will never return NULL unless new fails...

wctpXmlRecipient*
wctpXmlResponseHeader::GetRecipient(void) 
{
    if(m_pRecipient == NULL)
    {
        m_pRecipient = new wctpXmlRecipient;
    }
    return m_pRecipient;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_onBehalfOfRecipientID attribute

const char * 
wctpXmlResponseHeader::GetOnBehalfOfRecipientID(void) const 
{
    return m_onBehalfOfRecipientID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_onBehalfOfRecipientID attribute

void  
wctpXmlResponseHeader::SetOnBehalfOfRecipientID(const char * newValue)
{
    m_onBehalfOfRecipientID = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_respondingToTimestamp attribute

const char * 
wctpXmlResponseHeader::GetRespondingToTimestamp(void) const 
{
    return m_respondingToTimestamp;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_respondingToTimestamp attribute

void  
wctpXmlResponseHeader::SetRespondingToTimestamp(const char * newValue)
{
    m_respondingToTimestamp = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_responseTimestamp attribute

const char * 
wctpXmlResponseHeader::GetResponseTimestamp(void) const 
{
    return m_responseTimestamp;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_responseTimestamp attribute

void  
wctpXmlResponseHeader::SetResponseTimestamp(const char * newValue)
{
    m_responseTimestamp = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_responseToMessageID attribute

const char * 
wctpXmlResponseHeader::GetResponseToMessageID(void) const 
{
    return m_responseToMessageID;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_responseToMessageID attribute

void  
wctpXmlResponseHeader::SetResponseToMessageID(const char * newValue)
{
    m_responseToMessageID = newValue;
}

