/* File:  wctpXmlSubmitClientResponse.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlSubmitClientResponse.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlSubmitClientResponse::wctpXmlSubmitClientResponse()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlSubmitClientResponse::wctpXmlSubmitClientResponse(
    const wctpXmlSubmitClientResponse &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlSubmitClientResponse::~wctpXmlSubmitClientResponse()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlSubmitClientResponse&
wctpXmlSubmitClientResponse::operator=(
    const wctpXmlSubmitClientResponse &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlSubmitClientResponse::Init(void)
{
    wctpXmlOperation::Init();

    m_OperationType = WCTP_XML_SUBMIT_CLIENT_RESPONSE;
    m_SubmitClientResponseType = WCTP_XML_UNDEFINED;
    m_pFailure = NULL;
    m_pClientSuccess = NULL;

    m_title = SUBMIT_CLIENT_RESPONSE_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;

}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlSubmitClientResponse::Copy(const wctpXmlSubmitClientResponse &rval)
{

    wctpXmlOperation::Copy(rval);

    //
    // Copy object type
    //

    m_SubmitClientResponseType = rval.m_SubmitClientResponseType;

    //
    // Copy pFailure object if available (maybe holds)
    //

    if(rval.m_pFailure != NULL)
    {
        m_pFailure = new wctpXmlFailure(
            *rval.m_pFailure);
    }
    else
    {
        m_pFailure = NULL;
    }

    //
    // Copy pClientSuccess object if available (maybe holds)
    //

    if(rval.m_pClientSuccess != NULL)
    {
        m_pClientSuccess = new wctpXmlClientSuccess(
            *rval.m_pClientSuccess);
    }
    else
    {
        m_pClientSuccess = NULL;
    }

}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlSubmitClientResponse::Free(void)
{
    wctpXmlOperation::Free();

    if(m_pFailure != NULL) 
    {
        delete m_pFailure;
        m_pFailure = NULL;
    }
    if(m_pClientSuccess != NULL) 
    {
        delete m_pClientSuccess;
        m_pClientSuccess = NULL;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Clear object (reset object as if it were just constructed)

void
wctpXmlSubmitClientResponse::Clear(void)
{
    Free();
    Init();
}


//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlSubmitClientResponse::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-SubmitClientResponse>");
    if(m_SubmitClientResponseType == WCTP_XML_FAILURE && 
       m_pFailure != NULL)
    {
        dest += m_pFailure->AssembleOperation(ctx);
    }
    if(m_SubmitClientResponseType == WCTP_XML_CLIENT_SUCCESS && 
       m_pClientSuccess != NULL)
    {
        dest += m_pClientSuccess->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-SubmitClientResponse>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse switch for choice among set of objects (must match one)

wctp_parse_t 
wctpXmlSubmitClientResponse::ParseObjectSwitch(
    XString& msg,
    XmlNode *pTree, 
    int& pos)
{
    wctp_parse_t res = WCTP_PARSE_ERROR;
    if(pos >= pTree->children.size())
    {
        msg = "An element is required inside wctp-SubmitClientResponse";
        return res;
    }
    XmlNode *pChild = pTree->children[pos];
    if(pChild->element == "wctp-Failure")
    {
        HoldFailure();
        pos++;
        res = m_pFailure->ParseObject(msg,pChild);
    }
    else if(pChild->element == "wctp-ClientSuccess")
    {
        HoldClientSuccess();
        pos++;
        res = m_pClientSuccess->ParseObject(msg,pChild);
    }
    else 
    {
        msg = "unexpected element |";
        msg += pChild->element;
        msg += "| found inside wctp-SubmitClientResponse";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlSubmitClientResponse::ParseObject(XString& msg,XmlNode *pTree)
{
    int pos = 0;

    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-SubmitClientResponse")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-SubmitClientResponse, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }

    //
    // make sure no attributes where specified 
    //

    if(pTree->attributes.size() > 0)
    {
        msg = "wctp-SubmitClientResponse does not have any attributes";
        return WCTP_PARSE_ERROR;
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-SubmitClientResponse does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    wctp_parse_t res;

    //
    // Load in an object from a set
    //

    res = ParseObjectSwitch(msg, pTree, pos);
    if(res != WCTP_PARSE_OKAY)
    { 
        return res; 
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-SubmitClientResponse element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Accessor to find which object type is contained

wctp_xml_t 
wctpXmlSubmitClientResponse::GetSubmitClientResponseType(void)
{
    return m_SubmitClientResponseType;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Failure object
// This function will never return NULL unless new fails...

wctpXmlFailure*
wctpXmlSubmitClientResponse::GetFailure(void) 
{
    if(m_pFailure == NULL)
    {
        m_pFailure = new wctpXmlFailure;
    }
    return m_pFailure;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold Failure object 

void
wctpXmlSubmitClientResponse::HoldFailure(void) 
{
    m_SubmitClientResponseType = WCTP_XML_FAILURE;
    if(m_pFailure == NULL)
    {
        m_pFailure = new wctpXmlFailure;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to ClientSuccess object
// This function will never return NULL unless new fails...

wctpXmlClientSuccess*
wctpXmlSubmitClientResponse::GetClientSuccess(void) 
{
    if(m_pClientSuccess == NULL)
    {
        m_pClientSuccess = new wctpXmlClientSuccess;
    }
    return m_pClientSuccess;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold ClientSuccess object 

void
wctpXmlSubmitClientResponse::HoldClientSuccess(void) 
{
    m_SubmitClientResponseType = WCTP_XML_CLIENT_SUCCESS;
    if(m_pClientSuccess == NULL)
    {
        m_pClientSuccess = new wctpXmlClientSuccess;
    }
}

