/* File:  wctpXmlSubmitHeader.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlSubmitHeader.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlSubmitHeader::wctpXmlSubmitHeader()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlSubmitHeader::wctpXmlSubmitHeader(
    const wctpXmlSubmitHeader &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlSubmitHeader::~wctpXmlSubmitHeader()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlSubmitHeader&
wctpXmlSubmitHeader::operator=(
    const wctpXmlSubmitHeader &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlSubmitHeader::Init(void)
{
    m_pOriginator = new wctpXmlOriginator;
    m_pMessageControl = new wctpXmlMessageControl;
    m_pRecipient = new wctpXmlRecipient;
    m_submitTimestamp = "";
}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlSubmitHeader::Copy(const wctpXmlSubmitHeader &rval)
{
    //
    // Copy pOriginator object if available (should always hold one)
    //

    if(rval.m_pOriginator != NULL)
    {
        m_pOriginator = new wctpXmlOriginator(
            *rval.m_pOriginator);
    }
    else
    {
        m_pOriginator = new wctpXmlOriginator;
    }

    //
    // Copy pMessageControl object if available (should always hold one)
    //

    if(rval.m_pMessageControl != NULL)
    {
        m_pMessageControl = new wctpXmlMessageControl(
            *rval.m_pMessageControl);
    }
    else
    {
        m_pMessageControl = new wctpXmlMessageControl;
    }

    //
    // Copy pRecipient object if available (should always hold one)
    //

    if(rval.m_pRecipient != NULL)
    {
        m_pRecipient = new wctpXmlRecipient(
            *rval.m_pRecipient);
    }
    else
    {
        m_pRecipient = new wctpXmlRecipient;
    }

    //
    // Copy member variables
    //

    m_submitTimestamp = rval.m_submitTimestamp;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlSubmitHeader::Free(void)
{
    if(m_pOriginator != NULL) 
    {
        delete m_pOriginator;
        m_pOriginator = NULL;
    }
    if(m_pMessageControl != NULL) 
    {
        delete m_pMessageControl;
        m_pMessageControl = NULL;
    }
    if(m_pRecipient != NULL) 
    {
        delete m_pRecipient;
        m_pRecipient = NULL;
    }
}




//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlSubmitHeader::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-SubmitHeader");
    dest += ctx.XmlAttribute(
        "submitTimestamp",m_submitTimestamp);
    dest += ctx.IndentText(">");
    if(m_pOriginator != NULL)
    {
        dest += m_pOriginator->AssembleOperation(ctx);
    }
    if(m_pMessageControl != NULL)
    {
        dest += m_pMessageControl->AssembleOperation(ctx);
    }
    if(m_pRecipient != NULL)
    {
        dest += m_pRecipient->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-SubmitHeader>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlSubmitHeader::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "submitTimestamp")
    {
        m_submitTimestamp = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-SubmitHeader";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlSubmitHeader::ParseObject(XString& msg, XmlNode *pTree)
{

    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-SubmitHeader")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-SubmitHeader, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-SubmitHeader does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    int pos = 0;
    wctp_parse_t res;

    //
    // Load in wctp-Originator object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pOriginator->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Originator element expected in ";
        msg += "wctp-SubmitHeader element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in wctp-MessageControl object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pMessageControl->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-MessageControl element expected in ";
        msg += "wctp-SubmitHeader element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Load in wctp-Recipient object
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos++];
        res = m_pRecipient->ParseObject(msg,pChild);
        if(res != WCTP_PARSE_OKAY)
        {
            return WCTP_PARSE_ERROR;
        }
    }
    else
    {
        msg = "wctp-Recipient element expected in ";
        msg += "wctp-SubmitHeader element";
        return WCTP_PARSE_ERROR;
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-SubmitHeader element does not accept a |";
        msg += pChild->element;
        msg += "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Originator object
// This function will never return NULL unless new fails...

wctpXmlOriginator*
wctpXmlSubmitHeader::GetOriginator(void) 
{
    if(m_pOriginator == NULL)
    {
        m_pOriginator = new wctpXmlOriginator;
    }
    return m_pOriginator;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to MessageControl object
// This function will never return NULL unless new fails...

wctpXmlMessageControl*
wctpXmlSubmitHeader::GetMessageControl(void) 
{
    if(m_pMessageControl == NULL)
    {
        m_pMessageControl = new wctpXmlMessageControl;
    }
    return m_pMessageControl;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Recipient object
// This function will never return NULL unless new fails...

wctpXmlRecipient*
wctpXmlSubmitHeader::GetRecipient(void) 
{
    if(m_pRecipient == NULL)
    {
        m_pRecipient = new wctpXmlRecipient;
    }
    return m_pRecipient;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_submitTimestamp attribute

const char * 
wctpXmlSubmitHeader::GetSubmitTimestamp(void) const 
{
    return m_submitTimestamp;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_submitTimestamp attribute

void  
wctpXmlSubmitHeader::SetSubmitTimestamp(const char * newValue)
{
    m_submitTimestamp = newValue;
}

