/* File:  wctpXmlVersionQuery.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlVersionQuery.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlVersionQuery::wctpXmlVersionQuery()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlVersionQuery::wctpXmlVersionQuery(
    const wctpXmlVersionQuery &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlVersionQuery::~wctpXmlVersionQuery()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlVersionQuery&
wctpXmlVersionQuery::operator=(
    const wctpXmlVersionQuery &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlVersionQuery::Init(void)
{
    wctpXmlOperation::Init();

    m_OperationType = WCTP_XML_VERSION_QUERY;

    m_inquirer = "";
    m_dateTime = "";
    m_wctpToken = "";

    m_title = VERSION_QUERY_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;


}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlVersionQuery::Copy(const wctpXmlVersionQuery &rval)
{
    wctpXmlOperation::Copy(rval);

    //
    // Copy member variables
    //

    m_inquirer = rval.m_inquirer;
    m_dateTime = rval.m_dateTime;
    m_wctpToken = rval.m_wctpToken;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlVersionQuery::Free(void)
{
    wctpXmlOperation::Free();
}

//////////////////////////////////////////////////////////////////////////////
// Clear object (reset object as if it were just constructed)

void
wctpXmlVersionQuery::Clear(void)
{
    Free();
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlVersionQuery::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    dest += ctx.OpenIndent("<wctp-VersionQuery");
    dest += ctx.XmlAttribute(
        "inquirer",m_inquirer);
    dest += ctx.XmlAttribute(
        "dateTime",m_dateTime);
    dest += ctx.XmlAttribute(
        "wctpToken",m_wctpToken);
    dest += ctx.CloseIndent("/>");
    return dest;     
}


//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlVersionQuery::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "inquirer")
    {
        m_inquirer = val;
    }
    else if(key == "dateTime")
    {
        m_dateTime = val;
    }
    else if(key == "wctpToken")
    {
        m_wctpToken = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-VersionQuery";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlVersionQuery::ParseObject(XString& msg,XmlNode *pTree)
{
    
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-VersionQuery")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-VersionQuery, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }


    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-VersionQuery does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Should not be any child objects
    //

    if(pTree->children.size() > 0)
    {
        msg = "wctp-VersionQuery element does not accept any child elements";
        return WCTP_PARSE_ERROR;
    }
    
    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_inquirer attribute

const char * 
wctpXmlVersionQuery::GetInquirer(void) const 
{
    return m_inquirer;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_inquirer attribute

void  
wctpXmlVersionQuery::SetInquirer(const char * newValue)
{
    m_inquirer = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_dateTime attribute

const char * 
wctpXmlVersionQuery::GetDateTime(void) const 
{
    return m_dateTime;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_dateTime attribute

void  
wctpXmlVersionQuery::SetDateTime(const char * newValue)
{
    m_dateTime = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_wctpToken attribute

const char * 
wctpXmlVersionQuery::GetWctpToken(void) const 
{
    return m_wctpToken;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_wctpToken attribute

void  
wctpXmlVersionQuery::SetWctpToken(const char * newValue)
{
    m_wctpToken = newValue;
}

