/* File:  wctpXmlVersionQueryResponse.cpp

The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.skytel.com/downloads/wctpxml/worldcom_mpl_1.1.txt.

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is the WCTP Toolkit

The Initial Developer of the Original Code is WorldCom, Inc.
Portions created by WorldCom, Inc. are Copyright  2001
WorldCom, Inc.  All Rights Reserved.

Contributor(s):
  WorldCom, Inc. <skytel-strategic-apps@wcom.com>  

*/

#include "wctpXmlVersionQueryResponse.hpp"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
// Constructor

wctpXmlVersionQueryResponse::wctpXmlVersionQueryResponse()
{
    Init();
}

//////////////////////////////////////////////////////////////////////////////
// Copy Constructor

wctpXmlVersionQueryResponse::wctpXmlVersionQueryResponse(
    const wctpXmlVersionQueryResponse &rval)
{
    Copy(rval);
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

wctpXmlVersionQueryResponse::~wctpXmlVersionQueryResponse()
{
    Free();
}

//////////////////////////////////////////////////////////////////////////////
// Assignment Operator

wctpXmlVersionQueryResponse&
wctpXmlVersionQueryResponse::operator=(
    const wctpXmlVersionQueryResponse &rval)
{
    Free();
    Copy(rval);
    return *this;
}

//////////////////////////////////////////////////////////////////////////////
// Initialize member elements to default values

void
wctpXmlVersionQueryResponse::Init(void)
{
    wctpXmlOperation::Init();

    m_OperationType = WCTP_XML_VERSION_QUERY_RESPONSE;

    m_VersionQueryResponseType = WCTP_XML_UNDEFINED;
    m_pContactInfo = NULL;
    m_pFailure = NULL;

    m_inquirer = "";
    m_responder = "";
    m_dateTimeReq = "";
    m_dateTimeRsp = "";
    m_invalidAfter = "";
    m_listDTD = "";
    m_listConfig = "";

    m_title = VERSION_RESPONSE_STRING;

    m_xmlOperation = m_xmlPrefix;
    m_xmlOperation += "-";
    m_xmlOperation += m_title;


}

//////////////////////////////////////////////////////////////////////////////
// Deep copy member elements

void
wctpXmlVersionQueryResponse::Copy(const wctpXmlVersionQueryResponse &rval)
{
    wctpXmlOperation::Copy(rval);

    //
    // Copy object type
    //

    m_VersionQueryResponseType = rval.m_VersionQueryResponseType;

    //
    // Copy pVersionContactInfo object if available (maybe holds)
    //

    if(rval.m_pContactInfo != NULL)
    {
        m_pContactInfo = new wctpXmlVersionContactInfo(
            *rval.m_pContactInfo);
    }
    else
    {
        m_pContactInfo = NULL;
    }

    //
    // Copy vDTDSupport object(s) if available (holds one or more)
    //

    if(rval.m_vDTDSupport.size() > 0)
    {
        int i;
        for(i=0; i<rval.m_vDTDSupport.size(); i++)
        {
            wctpXmlVersionDTDSupport *pDTDSupport = 
                new wctpXmlVersionDTDSupport(
                *(rval.m_vDTDSupport[i]));
            m_vDTDSupport.push(pDTDSupport);
        }
    }

    //
    // Copy pFailure object if available (maybe holds)
    //

    if(rval.m_pFailure != NULL)
    {
        m_pFailure = new wctpXmlFailure(
            *rval.m_pFailure);
    }
    else
    {
        m_pFailure = NULL;
    }

    //
    // Copy member variables
    //

    m_inquirer = rval.m_inquirer;
    m_responder = rval.m_responder;
    m_dateTimeReq = rval.m_dateTimeReq;
    m_dateTimeRsp = rval.m_dateTimeRsp;
    m_invalidAfter = rval.m_invalidAfter;
    m_listDTD = rval.m_listDTD;
    m_listConfig = rval.m_listConfig;
}

//////////////////////////////////////////////////////////////////////////////
// Free any objects being held

void
wctpXmlVersionQueryResponse::Free(void)
{
    wctpXmlOperation::Free();

    m_vDTDSupport.destroy();

    if(m_pContactInfo != NULL) 
    {
        delete m_pContactInfo;
        m_pContactInfo = NULL;
    }

    if(m_pFailure != NULL) 
    {
        delete m_pFailure;
        m_pFailure = NULL;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Clear object (reset object as if it were just constructed)

void
wctpXmlVersionQueryResponse::Clear(void)
{
    Free();
    Init();
}


//////////////////////////////////////////////////////////////////////////////
// Construct an XML string for VersionContactInfo object(s)

XString
wctpXmlVersionQueryResponse::AssembleVersionDTDSupport(XmlContext& ctx)
{
    XString dest;
    int i;
    for(i=0; i<m_vDTDSupport.size(); i++) 
    {
        dest += m_vDTDSupport[i]->AssembleOperation(ctx); 
    }
    return dest;
}

//////////////////////////////////////////////////////////////////////////////
// Construct XML string for this object using given context 

XString 
wctpXmlVersionQueryResponse::AssembleOperation(XmlContext& ctx)
{
    XString dest;
    int i;

    dest += ctx.OpenIndent("<wctp-VersionResponse");
    dest += ctx.XmlAttribute(
        "responder",m_responder);
    dest += ctx.XmlAttribute(
        "dateTimeOfRsp",m_dateTimeRsp);
    dest += ctx.XmlAttribute(
        "inquirer",m_inquirer);
    dest += ctx.XmlAttribute(
        "dateTimeOfReq",m_dateTimeReq);
    dest += ctx.XmlAttribute(
        "invalidAfter",m_invalidAfter);
    //dest += ctx.XmlAttribute(
    //    "listDTDs",m_listDTD);
    //dest += ctx.XmlAttribute(
    //    "listConfiguration",m_listConfig);    
    dest += ctx.IndentText(">");

    if(m_VersionQueryResponseType == WCTP_XML_VERSION_CONTACT_INFO &&
       m_pContactInfo != NULL)
    {
        dest += m_pContactInfo->AssembleOperation(ctx);
    }

    if(m_VersionQueryResponseType == WCTP_XML_VERSION_DTD_SUPPORT)
    {
        for (i=0; i<m_vDTDSupport.size(); i++)
        {
            dest += AssembleVersionDTDSupport(ctx);
        }
    }

    if(m_VersionQueryResponseType == WCTP_XML_FAILURE && 
       m_pFailure != NULL)
    {
        dest += m_pFailure->AssembleOperation(ctx);
    }
    dest += ctx.CloseIndent("</wctp-VersionResponse>");
    return dest;     
}

//////////////////////////////////////////////////////////////////////////////
// Parse Attribute for this object

wctp_parse_t 
wctpXmlVersionQueryResponse::ParseAttribute(
    XString& msg,
    const XString& key, 
    const XString& val)
{
    wctp_parse_t res = WCTP_PARSE_OKAY;
    if(key == "responder")
    {
        m_responder = val;
    }
    else if(key == "dateTimeOfRsp")
    {
        m_dateTimeRsp = val;
    }
    else if(key == "inquirer")
    {
        m_inquirer = val;
    }
    else if(key == "dateTimeOfReq")
    {
        m_dateTimeReq = val;
    }
    else if(key == "invalidAfter")
    {
        m_invalidAfter = val;
    }
    else if(key == "listDTDs")
    {
        m_listDTD = val;
    }
    else if(key == "listConfiguration")
    {
        m_listConfig = val;
    }
    else
    {
        msg = "attribute |";
        msg += key;
        msg += "| does not exist for element wctp-VersionResponse";
        res = WCTP_PARSE_ERROR;
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse one or more VersionDTDSupport object(s)

wctp_parse_t
wctpXmlVersionQueryResponse::ParseVersionDTDSupport(
    XString& msg,
    XmlNode *pTree,
    int& pos)
{  
    bool flag = false;
    while(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        if(pChild->element != "wctp-DTDSupport")
        {
            break;
        }
        flag = true;
        pos++;
        wctpXmlVersionDTDSupport *pDTDSupport = 
                    AddVersionDTDSupport();
        wctp_parse_t res = 
                pDTDSupport->ParseObject(msg, pChild); 
        if(res != WCTP_PARSE_OKAY)
        {
            return res; 
        }
    }
    if(!flag) 
    {
        msg = "expect at least one wctp-DTDSupport ";
        msg += "inside wctp-VersionResponse";
        return WCTP_PARSE_ERROR;        
    }
    
    return WCTP_PARSE_OKAY;

    /*
    XmlNode *pChild = pTree->children[pos];
    if(pChild->element == "wctp-DTDSupport")
    {
        pos ++;
        wctpXmlVersionDTDSupport *pDTDSupport = AddVersionDTDSupport();
        wctp_parse_t res = pDTDSupport->ParseObject(msg, pChild); 
        if(res != WCTP_PARSE_OKAY)
        {
            return res; 
        }
    }
    else    
    {
        msg = "expect at least one wctp-DTDSuport ";
        msg += "inside " + m_xmlOperation;
        return WCTP_PARSE_ERROR;        
    }
    return WCTP_PARSE_OKAY;
    */   
}

//////////////////////////////////////////////////////////////////////////////
// Parse switch for choice among set of objects (must match one)

wctp_parse_t 
wctpXmlVersionQueryResponse::ParseObjectSwitch(
    XString& msg,
    XmlNode *pTree, 
    int& pos)
{
    wctp_parse_t res = WCTP_PARSE_ERROR;
    if(pos >= pTree->children.size())
    {
        msg = "An element is required inside wctp-VersionResponse";
        return res;
    }

    while (pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        if(pChild->element == "wctp-ContactInfo")
        {
            HoldVersionContactInfo();
            pos++;
            res = m_pContactInfo->ParseObject(msg,pChild);
        }
        else if(pChild->element == "wctp-DTDSupport")
        {
            HoldVersionDTDSupport();
            //pos++;
            //res = ParseVersionDTDSupport(msg,pTree,pos);
            //m_VersionQueryResponseType = WCTP_XML_VERSION_DTD_SUPPORT;
            res = ParseVersionDTDSupport(msg, pTree, pos);
        }
        else if(pChild->element == "wctp-Failure")
        {
            HoldFailure();
            pos++;
            res = m_pFailure->ParseObject(msg,pChild);
        }
        else 
        {
            msg = "unexpected element |";
            msg += pChild->element;
            msg += "| found inside wctp-VersionResponse";
            res = WCTP_PARSE_ERROR;
            break;
        }
    }
    return res;
}

//////////////////////////////////////////////////////////////////////////////
// Parse XML tree for this object 

wctp_parse_t 
wctpXmlVersionQueryResponse::ParseObject(XString& msg,XmlNode *pTree)
{
    int pos = 0;
    
    //
    // make sure the element matches me
    //

    if(pTree->element != "wctp-VersionResponse")
    {
        msg = "element mismatch, ";
        msg += "expected wctp-VersionResponse, but got |";
        msg += pTree->element;
        msg += "|";
        return WCTP_PARSE_ERROR;  
    }

    //
    // load in attribute values
    //

    int i;
    for(i=0; i<pTree->attributes.size(); i++)
    {
        XmlAttribute *pAttribute = pTree->attributes[i];
        wctp_parse_t res = ParseAttribute(
            msg, pAttribute->key, pAttribute->val);
        if(res != WCTP_PARSE_OKAY)
        {
            return res;
        }
    }

     //
     // make sure no pcdata
     //

     XString pcdata = pTree->pcdata.both_trim();
     if(pcdata.size() > 0)
     {
        msg = "wctp-VersionResponse does not take pcdata";
        return WCTP_PARSE_ERROR;
     }

    //
    // Load in child objects
    //

    wctp_parse_t res;

    //
    // Load in an object from a set
    //

    res = ParseObjectSwitch(msg, pTree, pos);
    if(res != WCTP_PARSE_OKAY)
    { 
        return res; 
    }

    //
    // Should not be any remaining child objects
    //

    if(pos < pTree->children.size())
    {
        XmlNode *pChild = pTree->children[pos];
        msg = "wctp-VersionResponse element does not accept a |";
        msg += pChild->element;
        msg+= "| child element";
        return WCTP_PARSE_ERROR;
    }

    return WCTP_PARSE_OKAY;
}

//////////////////////////////////////////////////////////////////////////////
// Accessor to find which object type is contained

wctp_xml_t 
wctpXmlVersionQueryResponse::GetVersionQueryResponseType(void)
{
    return m_VersionQueryResponseType;
}

//////////////////////////////////////////////////////////////////////////////
// Clear all VersionDTDSupport elements

void
wctpXmlVersionQueryResponse::ClearVersionDTDSupport(void) 
{
    m_vDTDSupport.destroy();
}

//////////////////////////////////////////////////////////////////////////////
// Add VersionDTDSupport element to end of array

wctpXmlVersionDTDSupport *
wctpXmlVersionQueryResponse::AddVersionDTDSupport(void)
{
    wctpXmlVersionDTDSupport *pDTDSupport = 
        new wctpXmlVersionDTDSupport;
    m_vDTDSupport.push(pDTDSupport);
    return pDTDSupport;
}

//////////////////////////////////////////////////////////////////////////////
// Erase VersionDTDSupport element at given position

void
wctpXmlVersionQueryResponse::EraseVersionDTDSupport(int pos)
{
    wctpXmlVersionDTDSupport *pDTDSupport = 
        m_vDTDSupport.remove(pos);
    delete pDTDSupport;
    pDTDSupport = NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Return number of VersionDTDSupport elements

int
wctpXmlVersionQueryResponse::NumVersionDTDSupport(void)
{
    return m_vDTDSupport.size();
}

//////////////////////////////////////////////////////////////////////////////
// Get DTDSupport element at given position

wctpXmlVersionDTDSupport*
wctpXmlVersionQueryResponse::GetVersionDTDSupport(int pos)
{
    if(pos >= 0 && pos < m_vDTDSupport.size())
    {
        return m_vDTDSupport[pos];
    }
    return NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold DTDSupport object 

void
wctpXmlVersionQueryResponse::HoldVersionDTDSupport(void)
{
    m_VersionQueryResponseType = WCTP_XML_VERSION_DTD_SUPPORT;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to VersionContactInfo object
// This function will never return NULL unless new fails...

wctpXmlVersionContactInfo *
wctpXmlVersionQueryResponse::GetVersionContactInfo(void)
{
    if(m_pContactInfo == NULL)
    {
        m_pContactInfo = new wctpXmlVersionContactInfo;
    }
    return m_pContactInfo;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold VersionContactInfo object 

void
wctpXmlVersionQueryResponse::HoldVersionContactInfo(void)
{
    m_VersionQueryResponseType = WCTP_XML_VERSION_CONTACT_INFO;

    if(m_pContactInfo == NULL)
    {
        m_pContactInfo = new wctpXmlVersionContactInfo;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for pointer to Failure object
// This function will never return NULL unless new fails...

wctpXmlFailure*
wctpXmlVersionQueryResponse::GetFailure(void) 
{
    if(m_pFailure == NULL)
    {
        m_pFailure = new wctpXmlFailure;
    }
    return m_pFailure;
}

//////////////////////////////////////////////////////////////////////////////
// Specify container to hold Failure object 

void
wctpXmlVersionQueryResponse::HoldFailure(void) 
{
    m_VersionQueryResponseType = WCTP_XML_FAILURE;
    if(m_pFailure == NULL)
    {
        m_pFailure = new wctpXmlFailure;
    }
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_responder attribute

const char * 
wctpXmlVersionQueryResponse::GetResponder(void) const 
{
    return m_responder;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_responder attribute

void  
wctpXmlVersionQueryResponse::SetResponder(const char * newValue)
{
    m_responder = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_dateTimeRsp attribute

const char * 
wctpXmlVersionQueryResponse::GetDateTimeOfRsp(void) const 
{
    return m_dateTimeRsp;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_dateTimeRsp attribute

void  
wctpXmlVersionQueryResponse::SetDateTimeOfRsp(const char * newValue)
{
    m_dateTimeRsp = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_inquirer attribute

const char * 
wctpXmlVersionQueryResponse::GetInquirer(void) const 
{
    return m_inquirer;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_inquirer attribute

void  
wctpXmlVersionQueryResponse::SetInquirer(const char * newValue)
{
    m_inquirer = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_dateTimeReq attribute

const char * 
wctpXmlVersionQueryResponse::GetDateTimeOfReq(void) const 
{
    return m_dateTimeReq;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_dateTimeReq attribute

void  
wctpXmlVersionQueryResponse::SetDateTimeOfReq(const char * newValue)
{
    m_dateTimeReq = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_invalidAfter attribute

const char * 
wctpXmlVersionQueryResponse::GetInvalidAfter(void) const 
{
    return m_invalidAfter;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_invalidAfter attribute

void  
wctpXmlVersionQueryResponse::SetInvalidAfter(const char * newValue)
{
    m_invalidAfter = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_listDTD attribute

const char * 
wctpXmlVersionQueryResponse::GetListDTD(void) const 
{
    return m_listDTD;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_listDTD attribute

void  
wctpXmlVersionQueryResponse::SetListDTD(const char * newValue)
{
    m_listDTD = newValue;
}

//////////////////////////////////////////////////////////////////////////////
// Access operation for m_listConfig attribute

const char * 
wctpXmlVersionQueryResponse::GetListConfig(void) const 
{
    return m_listConfig;
}

//////////////////////////////////////////////////////////////////////////////
// Settor operation for m_listConfig attribute

void  
wctpXmlVersionQueryResponse::SetListConfig(const char * newValue)
{
    m_listConfig = newValue;
}
